/*
**
**                                 A S Y . C
**
**
**      Module:     asy statistic support
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      20-OCT-1992 AV      Version X1.0, first definition
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#if defined(MODULE_ASY)

#include <stdlib.h>
#include <curses.h>
#include <panel.h>

#include <sys/types.h>
#include <sys/stream.h>
#include <sys/pp_dep.h>
#include <sys/queue.h>
#include <sys/strppc.h>
#include <sys/strtty.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  Port: xx In: ######### Out: ######### Rdqp: ######### Device: ####
**  state: TIMEOUT WOPEN ISOPEN TBLOCK CARR_ON WIOC WGETTY TTSTOP EXTPROC TACT CLESC
**         RTO TTIOW TTXON TTXOFF BUSY
**  dstat: SETOPT SPLITMSG WENTRY SUPBUF WRTQDRAIN CLDRAIN OPDRAIN WBREAK
**  cc:    intr ## quit ## erase ## kill ## eof ## eol ## eol2 ## swtch ## lnext ##
**         start ## stop ## susp ## dsusp ## reprint ## discard ## werase ##
**  iflag: IGNBRK BRKINT IGNPAR PARMRK INPCK ISTRIP INLCR IGNCR ICRNL IUCLC
**         IXON   IXANY  IXOFF
**  oflag: OPOST OLCUC ONLCR OCRNL ONOCR ONLRET OFILL OFDEL NL# CR# TAB# BS# VT# FF#
**  lflag: ISIG ICANON XCASE ECHO ECHOE ECHOK ECHONL NOFLSH
**  cflag:  38400-8-N-1 CLOCAL CREAD HUPCL
*/

#define ADDR_asy_cnt            kmemSymbols[0].n_value
#define ADDR_asy_tty            kmemSymbols[1].n_value

static struct nlist kmemSymbols[] =
{
#if defined(S_SINIX)
    { "asy_cnt"             },
#else
    { "num_asy"             },
#endif
    { "asy_tty"             },
    { 0                     },
};

static int      asyPort = 0;
static int      noSyms;

#define LASTPORT    9999

typedef struct slabel
{
    int     y;
    int     x;
    char    *label;
} SLABEL;

static SLABEL asyLabels[] =
{
    {   0,  0, "Port:"          },
    {   0,  9, "In:"            },
    {   0, 23, "Out:"           },
    {   0, 38, "Rdqp:"          },
    {   0, 54, "Device:"        },
    {   1,  0, "state:"         },
    {   3,  0, "dstat:"         },
    {   4,  0, "cc:"            },
    {   6,  0, "iflag:"         },
    {   8,  0, "oflag:"         },
    {   9,  0, "lflag:"         },
    {  10,  0, "cflag:"         },
    {  -1, -1, (char *) 0       }
};

typedef struct bitfld
{
    int     y;
    int     x;
    char    *label;
    int     flag_num;
    uint    mask;
} BITFLD;

#define IFLAG   1
#define CFLAG   2
#define OFLAG   3
#define LFLAG   4
#define STATE   5
#define DSTAT   6

static BITFLD ttybitflds[] =
{
    {   0,   7,   "TIMEOUT",   STATE, TIMEOUT                  },
    {   0,  15,     "WOPEN",   STATE, WOPEN                    },
    {   0,  21,    "ISOPEN",   STATE, ISOPEN                   },
    {   0,  28,    "TBLOCK",   STATE, TBLOCK                   },
    {   0,  35,   "CARR_ON",   STATE, CARR_ON                  },
    {   0,  43,      "WIOC",   STATE, WIOC                     },
    {   0,  48,    "WGETTY",   STATE, WGETTY                   },
    {   0,  55,    "TTSTOP",   STATE, TTSTOP                   },
    {   0,  62,   "EXTPROC",   STATE, EXTPROC                  },
    {   0,  70,      "TACT",   STATE, TACT                     },
    {   0,  75,     "CLESC",   STATE, CLESC                    },

    {   1,   7,       "RTO",   STATE, RTO                      },
    {   1,  11,     "TTIOW",   STATE, TTIOW                    },
    {   1,  17,     "TTXON",   STATE, TTXON                    },
    {   1,  23,    "TTXOFF",   STATE, TTXOFF                   },
    {   1,  30,      "BUSY",   STATE, BUSY                     },

    {   2,   7,    "SETOPT",   DSTAT, SETOPT                   },
    {   2,  14,  "SPLITMSG",   DSTAT, SPLITMSG                 },
    {   2,  23,    "WENTRY",   DSTAT, WENTRY                   },
    {   2,  30,    "SUPBUF",   DSTAT, SUPBUF                   },
    {   2,  37, "WRTQDRAIN",   DSTAT, WRTQDRAIN                },
    {   2,  47,   "CLDRAIN",   DSTAT, CLDRAIN                  },
    {   2,  55,   "OPDRAIN",   DSTAT, OPDRAIN                  },
    {   2,  63,    "WBREAK",   DSTAT, WBREAK                   },

    {   5,   7,    "IGNBRK",   IFLAG, IGNBRK                   },
    {   5,  14,    "BRKINT",   IFLAG, BRKINT                   },
    {   5,  21,    "IGNPAR",   IFLAG, IGNPAR                   },
    {   5,  28,    "PARMRK",   IFLAG, PARMRK                   },
    {   5,  35,     "INPCK",   IFLAG, INPCK                    },
    {   5,  41,    "ISTRIP",   IFLAG, ISTRIP                   },
    {   5,  48,     "INLCR",   IFLAG, INLCR                    },
    {   5,  54,     "IGNCR",   IFLAG, IGNCR                    },
    {   5,  60,     "ICRNL",   IFLAG, ICRNL                    },
    {   5,  66,     "IUCLC",   IFLAG, IUCLC                    },

    {   6,   7,      "IXON",   IFLAG, IXON                     },
    {   6,  14,     "IXANY",   IFLAG, IXANY                    },
    {   6,  21,     "IXOFF",   IFLAG, IXOFF                    },

    {   7,   7,     "OPOST",   OFLAG, OPOST                    },
    {   7,  13,     "OLCUC",   OFLAG, OLCUC                    },
    {   7,  19,     "ONLCR",   OFLAG, ONLCR                    },
    {   7,  25,     "OCRNL",   OFLAG, OCRNL                    },
    {   7,  31,     "ONOCR",   OFLAG, ONOCR                    },
    {   7,  37,    "ONLRET",   OFLAG, ONLRET                   },
    {   7,  44,     "OFILL",   OFLAG, OFILL                    },
    {   7,  50,     "OFDEL",   OFLAG, OFDEL                    },

    {   8,   7,      "ISIG",   LFLAG, ISIG                     },
    {   8,  12,    "ICANON",   LFLAG, ICANON                   },
    {   8,  19,     "XCASE",   LFLAG, XCASE                    },
    {   8,  25,      "ECHO",   LFLAG, ECHO                     },
    {   8,  30,     "ECHOE",   LFLAG, ECHOE                    },
    {   8,  36,     "ECHOK",   LFLAG, ECHOK                    },
    {   8,  42,    "ECHONL",   LFLAG, ECHONL                   },
    {   8,  49,    "NOFLSH",   LFLAG, NOFLSH                   },

    {   9,  20,    "CLOCAL",   CFLAG, CLOCAL                   },
    {   9,  27,     "CREAD",   CFLAG, CREAD                    },
    {   9,  33,     "HUPCL",   CFLAG, HUPCL                    },

    {   -1, -1,  (char *) 0,      -1, -1                       }
};

static char *speeds[] =
{
    "  HUP", "   50", "   75", "  110", "  134", "  150", "  200",
    "  300", "  600", " 1200", " 1800", " 2400", " 4800", " 9600",
    "19200", "38400"
};

static char *BuildDps (uint cflag)
{
    static char     rtnstr[64];
    register char   *cptr;

    if ((cflag & CBAUD) >= 0 && (cflag & CBAUD) < sizeof (speeds))
	(void) strcpy (rtnstr, speeds[cflag & CBAUD]);
    else
	(void) strcpy (rtnstr, "?????");

    cptr = rtnstr + strlen (rtnstr);
    *cptr++ = '-';

    switch (cflag & CSIZE)
    {
	case CS5: *cptr++ = '5'; break;
	case CS6: *cptr++ = '6'; break;
	case CS7: *cptr++ = '7'; break;
	case CS8: *cptr++ = '8'; break;
    }
    *cptr++ = '-';
    *cptr++ = (cflag & PARENB) ? ((cflag & PARODD) ? 'O' : 'E') : 'N';
    *cptr++ = '-';
    *cptr++ = (cflag & CSTOPB) ? '2' : '1';
    *cptr   = '\0';

    return (rtnstr);
}


static void ShowAsyEntry (
    WINDOW                  *win,
    int                     port,
    int                     tly,
    int                     tlx,
    struct strtty           *tip
)
{
    BITFLD      *bfptr = ttybitflds;
    SLABEL      *sptr  = asyLabels;
    uint        flag;

#ifdef NOTDEF
    if (!tip->t_state)
	return;
#endif

    tly++;

    use_cp (win, cpLIT);

    while (sptr->y >= 0)
    {
	wmove (win, sptr->y + tly, sptr->x + tlx);
	waddstr (win, sptr->label);
	sptr++;
    }

    use_cp (win, cpINFO);
    mvwprintw (win, tly + 0, tlx +  6, "%2d", port);
    mvwprintw (win, tly + 0, tlx + 13, "%8x", tip->t_in);
    mvwprintw (win, tly + 0, tlx + 28, "%8x", tip->t_out);
    mvwprintw (win, tly + 0, tlx + 44, "%8x", tip->t_rdqp);
    mvwprintw (win, tly + 0, tlx + 62, "%4u", tip->t_dev);

    mvwaddstr (win, tly + 10, tlx + 8, BuildDps (tip->t_cflag));

    while (bfptr->y >= 0)
    {
	switch (bfptr->flag_num)
	{
	    case IFLAG: flag = tip->t_iflag; break;
	    case CFLAG: flag = tip->t_cflag; break;
	    case OFLAG: flag = tip->t_oflag; break;
	    case LFLAG: flag = tip->t_lflag; break;
	    case STATE: flag = tip->t_state; break;
	    case DSTAT: flag = tip->t_dstat; break;
	}

	flag &= bfptr->mask;

	wmove (win, bfptr->y + tly + 1, bfptr->x + tlx);

	if (flag)
	    use_cp (win, cpREVERSE);

	waddstr (win, bfptr->label);

	if (flag)
	    use_cp (win, cpINFO);

	bfptr++;
    }

    use_cp (win, cpINFO);

    mvwprintw (win, tly + 4, tlx +  7, "intr %02x", tip->t_cc[VINTR]);
    mvwprintw (win, tly + 4, tlx + 15, "quit %02x", tip->t_cc[VQUIT]);
    mvwprintw (win, tly + 4, tlx + 23, "erase %02x", tip->t_cc[VERASE]);
    mvwprintw (win, tly + 4, tlx + 32, "kill %02x", tip->t_cc[VKILL]);
    mvwprintw (win, tly + 4, tlx + 40, "eof %02x", tip->t_cc[VEOF]);
    mvwprintw (win, tly + 4, tlx + 47, "eol %02x", tip->t_cc[VEOL]);
    mvwprintw (win, tly + 4, tlx + 54, "eol2 %02x", tip->t_cc[VEOL2]);
    mvwprintw (win, tly + 4, tlx + 62, "swtch %02x", tip->t_cc[VSWTCH]);
    mvwprintw (win, tly + 4, tlx + 71, "lnext %02x", tip->t_cc[VLNEXT]);

    mvwprintw (win, tly + 5, tlx + 7,  "start %02x", tip->t_cc[VSTART]);
    mvwprintw (win, tly + 5, tlx + 16, "stop %02x", tip->t_cc[VSTOP]);
    mvwprintw (win, tly + 5, tlx + 24, "susp %02x", tip->t_cc[VSUSP]);
    mvwprintw (win, tly + 5, tlx + 32, "dsusp %02x", tip->t_cc[VDSUSP]);
    mvwprintw (win, tly + 5, tlx + 41, "reprint %02x", tip->t_cc[VREPRINT]);
    mvwprintw (win, tly + 5, tlx + 52, "discard %02x", tip->t_cc[VDISCARD]);
    mvwprintw (win, tly + 5, tlx + 63, "werase %02x", tip->t_cc[VWERASE]);

    if ((tip->t_oflag & NLDLY)  == NL0)
	mvwaddstr (win, tly + 8, tlx + 56, "NL0");
    else if ((tip->t_oflag & NLDLY)  == NL1)
	mvwaddstr (win, tly + 8, tlx + 56, "NL1");
    else
	mvwaddstr (win, tly + 8, tlx + 56, "NL?");

    if ((tip->t_oflag & CRDLY)  == CR0)
	mvwaddstr (win, tly + 8, tlx + 60, "CR0");
    else if ((tip->t_oflag & CRDLY)  == CR1)
	mvwaddstr (win, tly + 8, tlx + 60, "CR1");
    else if ((tip->t_oflag & CRDLY)  == CR2)
	mvwaddstr (win, tly + 8, tlx + 60, "CR2");
    else if ((tip->t_oflag & CRDLY)  == CR3)
	mvwaddstr (win, tly + 8, tlx + 60, "CR3");
    else
	mvwaddstr (win, tly + 8, tlx + 60, "CR?");

    if ((tip->t_oflag & TABDLY) == TAB0)
	mvwaddstr (win, tly + 8, tlx + 64, "TAB0");
    else if ((tip->t_oflag & TABDLY) == TAB1)
	mvwaddstr (win, tly + 8, tlx + 64, "TAB1");
    else if ((tip->t_oflag & TABDLY) == TAB2)
	mvwaddstr (win, tly + 8, tlx + 64, "TAB2");
    else if ((tip->t_oflag & TABDLY) == TAB3)
	mvwaddstr (win, tly + 8, tlx + 64, "TAB3");
    else
	mvwaddstr (win, tly + 8, tlx + 64, "TAB?");

    if ((tip->t_oflag & BSDLY)  == BS0)
	mvwaddstr (win, tly + 8, tlx + 69, "BS0");
    else if ((tip->t_oflag & BSDLY)  == BS1)
	mvwaddstr (win, tly + 8, tlx + 69, "BS1");
    else
	mvwaddstr (win, tly + 8, tlx + 69, "BS?");

    if ((tip->t_oflag & VTDLY)  == VT0)
	mvwaddstr (win, tly + 8, tlx + 73, "VT0");
    else if ((tip->t_oflag & VTDLY)  == VT1)
	mvwaddstr (win, tly + 8, tlx + 73, "VT1");
    else
	mvwaddstr (win, tly + 8, tlx + 73, "VT?");

    if ((tip->t_oflag & FFDLY)  == FF0)
	mvwaddstr (win, tly + 8, tlx + 77, "FF0");
    else if ((tip->t_oflag & FFDLY)  == FF1)
	mvwaddstr (win, tly + 8, tlx + 77, "FF1");
    else
	mvwaddstr (win, tly + 8, tlx + 77, "FF?");
}


void AsyInit (void)
{
    noSyms = GetSymbols (kmemSymbols, 0);
}


void AsyCommand (int cmd)
{
    if (noSyms)
    {
	beep ();
	return;
    }

    if (cmd >= '0' && cmd <= '9')
	asyPort = cmd - '0';
    else if (cmd == ',' || cmd == KEY_UP || cmd == KEY_PREVIOUS)
	asyPort -= (asyPort > 0) ? 1 : 0;
    else if (cmd == '.' || cmd == KEY_DOWN || cmd == KEY_NEXT)
	asyPort += 1;
    else if (cmd == '<' || cmd == KEY_LEFT || cmd == KEY_BEG)
	asyPort = 0;
    else if (cmd == '>' || cmd == KEY_RIGHT || cmd == KEY_END)
	asyPort = LASTPORT;
    else
    {
	asyPort = 0;
	beep ();
    }
}


void DspAsyLiterals (WINDOW *win, int tly, int tlx)
{
    static char *header = "---- ASY statistics ";
    int         x2;
    int         y;

    wmove (win, tly, tlx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x2);
    while (x2++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "ASY related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    use_cp (win, cpLIT);
    ClearArea (win, getmaxy (win) - 1, 0, getmaxx (win));
}


void DspAsy (WINDOW *win, int tly, int tlx)
{
    struct strtty           *tip;
    int                     count;
    int                     idx;
    int                     num;
    int                     entries;

    if (noSyms)
	return;

    count = *KTUT (ADDR_asy_cnt, int *);
    tip   = KTUT (ADDR_asy_tty, struct strtty *);

    entries = getmaxy (win) / 12;
    entries = min (count, entries);

    if (asyPort >= count)
	asyPort = count - 1;

    wclrtobot (win);

    for (num = idx = 0; num < entries && idx < count; idx++, tip++)
    {
	if (idx < asyPort)
	    continue;

	ShowAsyEntry (win, idx, tly + (10 * num), tlx, tip);

	num++;
    }

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

#else
/* Prevent "empty translation unit" warnings from the idiots at X3J11. */
static char ansi_c_idiots = 69;
#endif
