/*
**
**                                B O O T . C
**
**
**      Module:     routines for bootinfo display
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      08-OCT-1992 AV      Version X1.0, first definition
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#if defined(MODULE_BOOT)

#include <curses.h>
#include <panel.h>

#include <sys/types.h>
#include <sys/fcntl.h>

#define AT386
#include <sys/bootinfo.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"


static char *bmemf_text (ulong flags)
{
    static char     hex_errant[16];
    ulong           orig_flags = flags;

    if (!flags)
	return("    ");

    switch (flags)
    {
	case B_MEM_KTEXT: return ("KTXT");
	case B_MEM_KDATA: return ("KDTA");
	case B_MEM_NODMA: return ("NODM");
    }

    (void) sprintf (hex_errant, "%04x", (ushort) orig_flags);

    return (hex_errant);
}

static char             bootBuffer[BOOTINFO_LOC + 512];
static struct bootinfo  *binfo = (struct bootinfo *) 0;
static int              noSyms;

void BootInit ()
{
    int     fd;

    if ((fd = open (MEM_FILE, O_RDONLY)) >= 0)
    {
	if (read (fd, bootBuffer, sizeof (bootBuffer)) == sizeof (bootBuffer))
	    binfo = (struct bootinfo *) (bootBuffer + BOOTINFO_LOC);
	else
	    noSyms = 1;
	close (fd);
    }
    else
	noSyms = 1;
}

void DspBoot (WINDOW *win, int y, int x)
{
    int             i;
    struct bootmem  *bptr;

    use_cp (win, cpBANNER);

    wmove (win, y++, x);
    waddstr (win, "------ Bootinfo ------");

    if (noSyms || binfo == (struct bootinfo *) 0)
    {
	use_cp (win, cpBANWARN);

	y += 4;
	wmove (win, y++, x);
	waddstr (win, "not available, sorry!!");
    }

    wmove (win, y++, x);
    DispStaticLong (win, "bflags   ", "%08lx     ", binfo->bootflags);

    y++;
    wmove (win, y++, x);
    waddstr (win, "memory available      ");

    bptr = &binfo->memavail[0];
    for (i = 0; i < binfo->memavailcnt; i++, bptr++)
    {

	wmove (win, y++, x);
	wprintw (win, "%08lx %08lx %s",
	    bptr->base, bptr->extent, bmemf_text (bptr->flags)
	);
    }

    y++;
    wmove (win, y++, x);
    waddstr (win, "memory used           ");

    bptr = &binfo->memused[0];
    for (i = 0; i < binfo->memusedcnt; i++, bptr++)
    {
	wmove (win, y++, x);
	wprintw (win, "%08lx %08lx %s",
	    bptr->base, bptr->extent, bmemf_text (bptr->flags)
	);
    }
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspBoot (WINDOW *win, int y, int x)
{
    use_cp (win, cpBANNER);

    wmove (win, y++, x);
    waddstr (win, "------ Bootinfo ------");

    use_cp (win, cpBANWARN);

    y += 4;
    wmove (win, y++, x);
    waddstr (win, "not available, sorry!!");
}

void BootInit ()
{
}

#endif
