/*
**
**                              C O N F I G . H
**
**
**      Module:     configuration flags
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      15-OCT-1992 AV  define ARCH_X86 for intel platforms
**      16-OCT-1992 AV  conditionalize TCP/IP support
**      20-OCT-1992 AV  added generic asy support
**      21-OCT-1992 AV  partial redesign
**      21-OCT-1992 AV  added generic SCSI support
**      23-OCT-1992 AV  Version 1.0
**      24-OCT-1992 AV  SLIP support
**      19-NOV-1992 AV  WOLLONGONG vs. LACHMAN distinction
**
*/

#ifndef _config_h_
#define _config_h_

/*
**  There are several parts of svr4mon depending on the architecture.
**  Currently only for 386/486 platforms exists such information.
**  If we're not running on any of one of them, svr4mon should be
**  able to just live with it and therefore should ommit the architecture
**  dependent stuff.
**
**  Define/undefine according to your system. Maybe more than one could
**  be defined.
**  This list could be extended according to new supported architectures.
**
*/
#define MODULE_BOOT
#define MODULE_HACK
#define MODULE_HARD
#define MODULE_STREAM
#define MODULE_SCSI
#define MODULE_SLIP
#define MODULE_TCP
#define MODULE_IP

/*
**  Define one of the following network features, if you have
**  turned on either of the MODULE_TCP, MODULE_IP. Otherwise
**  makes no difference. If you are unsure; try LACHMAN first.
**
**  If you define SNMP. (depending on how your system TCP/IP
**  was compiled), you will get more variables shown.
**  Usually keep it defined.
*/
#define NETWORK_LACHMAN
#undef  NETWORK_WOLLONGONG
#define	SNMP

#if defined(NETWORK_WOLLONGONG) && defined(NETWORK_LACHMAN)
#   error   "CONFIG ERROR: you must not define NETWORK_WOLLONGONG and NETWORK_LACHMAN!!!"
#endif

/*
**  Define one of them or undefine all. They are mutally exclusive.
**  So don't define more than one of them!
**
**  FAS: the fas support provided here is not widely useful. It is trimmed
**       for a own port of fas to SVR4. Though it is very likely that other
**       FAS version will just do it without any or small changes.
**
**  SAS: currently version 1.25 is supported.
**
**  ASY: use this for the dumb asy driver. Not well tested!!
**       One difficulty for asy support is the fact, that the naming
**       convention aren't standardized.
**       So take a look to /stand/unix with nm and look for the following
**       two (type) vars. Then change the definitions in asy.c accordingly.
**
**       count:
**          asy_cnt     [SINIX-D 5.41]
**          asy_count
**          num_asy     [Consensys SVR4 4.0.3, 1.4 release]
**                      [ESIX SVR4.0.3a]
**                      [GENERICS]
**                      [Microport SVR4.0.3]
**          numasy
**          Nasy
**          NUMasy
**
**       array of struct strtty structures:
**          asy_tty
*/
#define MODULE_FAS
#undef  MODULE_SAS
#undef  MODULE_ASY

#if defined(MODULE_FAS)
#   if defined(MODULE_SAS) || defined(MODULE_ASY)
#       error   "CONFIG ERROR: you must not define MULTIPLES (fas,sas,asy)!!!"
#   endif
#elif defined(MODULE_SAS)
#   if defined(MODULE_FAS) || defined(MODULE_ASY)
#       error   "CONFIG ERROR: you must not define MULTIPLES (fas,sas,asy)!!!"
#   endif
#elif defined(MODULE_ASY)
#   if defined(MODULE_SAS) || defined(MODULE_FAS)
#       error   "CONFIG ERROR: you must not define MULTIPLES (fas,sas,asy)!!!"
#   endif
#endif

/*
**  The manual says that defining these symbols may speed up curses.
**  I don't know (yet). Define/undefine them just as you want.
*/
#undef  CURS_PERFORMANCE
#undef  CURS_MACROS

#endif
