/*
**
**                             D S P U T I L . C
**
**
**      Module:     display utilities
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV      Version X1.0, first definition
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#include <curses.h>
#include <panel.h>
#include <sys/types.h>

#include "svr4mon.h"
#include "dsputil.h"

#if defined(COLOR_PAIR)
long color_attr[] =
{
    COLOR_PAIR (0),
    COLOR_PAIR (cpINFO),
    COLOR_PAIR (cpREVERSE),
    COLOR_PAIR (cpBANWARN),
    COLOR_PAIR (cpLOW) | A_BOLD,
    COLOR_PAIR (cpMED) | A_BOLD,
    COLOR_PAIR (cpHIGH),
    COLOR_PAIR (cpBANNER),
    COLOR_PAIR (cpLIT),
    0
};
#endif

long mono_attr[] =
{
    0,                  /* 0 */
    0,                  /* cpINFO */
    A_REVERSE,          /* cpREVERSE */
    A_BOLD,             /* cpBANWARN */
    0,                  /* cpLOW */
    A_UNDERLINE,        /* cpMED */
    A_BLINK,            /* cpHIGH */
    A_REVERSE,          /* cpBANNER */
    A_DIM,              /* cpLIT */
};

void ClearAreaChar (WINDOW *win, int y, int x, int len, u_char fillchar)
{
    wmove (win, y, x);
    while (len-- > 0)
	waddch (win, (chtype) fillchar);
    wmove (win, y, x);
}

void ClearArea (WINDOW *win, int y, int x, int len)
{
    ClearAreaChar (win, y, x, len, ' ');
}

void ClearRectangle (WINDOW *win, int y1, int x1, int y2, int x2)
{
    int     y;
    int     x;

    for (y = y1; y <= y2; y++)
	for (x = x1; x <= x2; x++)
	{
	    wmove (win, y, x);
	    waddch (win, (chtype) ' ');
	}

    wmove (win, y1, x1);
}


void PanelFlush (void)
{
    wmove (wscr, getmaxy (wscr) - 1, getmaxx (wscr) - 1);
    update_panels ();
    curs_set (0);
    doupdate ();
    curs_set (1);
}

void WpError (WINDOW *win, char *desc)
{
    extern int  errno;
    extern int  sys_nerr;
    extern char *sys_errlist[];

    waddstr (win, desc);
    waddstr (win, ": ");

    if (errno < sys_nerr)
	waddstr (win, sys_errlist[errno]);
    else
	wprintw (win, "error %u", errno);
}

PANEL *MakePanel (int rows, int cols, int tly, int tlx, char *userp)
{
    WINDOW  *win = newwin (rows, cols, tly, tlx);
    PANEL   *pan;

    if (!win)
	return ((PANEL *) 0);

    if (pan = new_panel (win))
    {
	set_panel_userptr (pan, userp);
	return (pan);
    }

    delwin (win);

    return ((PANEL *) 0);
}

void DispInfoLong (WINDOW *win, char *label, char *fmt, long value)
{
    use_cp (win, cpLIT);
    waddstr (win, label);
    use_cp (win, cpINFO);
    wprintw (win, fmt, value);
}

void DispInfoInt (WINDOW *win, char *label, char *fmt, int value)
{
    use_cp (win, cpLIT);
    waddstr (win, label);
    use_cp (win, cpINFO);
    wprintw (win, fmt, value);
}

void DispStaticLong (WINDOW *win, char *label, char *fmt, long value)
{
    use_cp (win, cpLIT);
    waddstr (win, label);
    wprintw (win, fmt, value);
}

void DispStaticInt (WINDOW *win, char *label, char *fmt, int value)
{
    use_cp (win, cpLIT);
    waddstr (win, label);
    wprintw (win, fmt, value);
}

void DispMessage (chtype cp, char *msg)
{
    int     y;
    int     x;

    wmove (wscr, MSG_TLY - 1, 0);
    use_cp (wscr, cp);
    waddstr (wscr, msg);
    getyx (wscr, y, x);

    while (x < getmaxx (wscr))
	waddch (wscr, (chtype) ' '), x++;
}
