/*
**
**                                 F A S . C
**
**
**      Module:     fas statistic support
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV      Version X1.0, first definition
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#if defined(MODULE_FAS)

#include <stdlib.h>
#include <curses.h>
#include <panel.h>
#include <termios.h>

/*
**  NOTE: Check carefully this two defines. They must be set to the same
**        value as for the kernel build. Just define the same symbols
**        that were defined for kernel compilation. Otherwise structure
**        sizes differs and invalid information will retrieved.
*/
#define MERGE386
#undef  HAVE_VPIX
#include <sys/fas.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  Port: xx Type: xxxxxxxx Units: xxxx Vec: xxxx Sioa: xxxx SioaOff: xxxx Mcb: xxxx
**  iflag: BRKINT DOSMODE ICRNL IGNBRK IGNCR IGNPAR IMAXBEL INLCR INPCK ISTRIP
**         IUCLC  IXANY   IXOFF IXON   PARMRK
**  cflag:  38400-8-N-1 CLOCAL CREAD HUPCL
**  device_flag: OPN LCK ENBL XBSY XBRK XLCK HANG DTIM GTIM CARR
**  o_state:     CLOS DLOUT GTTY WOPN CCOO FCO UNEN CL HWHS
**  flow_flag:   HWOHS HWIHS HWOSTP HWISTP SWOSTP SWISTP SWFCREQ RXFER IBLK
**  mod_stat: CTS_DLT DSR_DLT RING_TDG DCD_DLT CTS_PRS DSR_PRS RING_PRS DCD_PRS
**  cstart: 0xXX cstop: 0xXX hupwait: XXXX
**
*/

#define ADDR_fas_cont               kmemSymbols[0].n_value
#define ADDR_fas_info_data          kmemSymbols[1].n_value
#define ADDR_fas_info_ptr           kmemSymbols[2].n_value
#define ADDR_fas_init_seq           kmemSymbols[3].n_value
#define ADDR_fas_int_ack            kmemSymbols[4].n_value
#define ADDR_fas_int_ack_port       kmemSymbols[5].n_value
#define ADDR_fas_mux_ack            kmemSymbols[6].n_value
#define ADDR_fas_mux_ack_port       kmemSymbols[7].n_value
#define ADDR_fas_physical_units     kmemSymbols[8].n_value
#define ADDR_fas_unit               kmemSymbols[9].n_value

static struct nlist kmemSymbols[] =
{
    { "fas_cont"            },
    { "fas_info_data"       },
    { "fas_info_ptr"        },
    { "fas_init_seq"        },
    { "fas_int_ack"         },
    { "fas_int_ack_port"    },
    { "fas_mux_ack"         },
    { "fas_mux_ack_port"    },
    { "fas_physical_units"  },
    { "fas_unit"            },
    { 0                     },
};

static int      fasPort = 0;
static int      noSyms;

#define LASTPORT    9999

typedef struct slabel
{
    int     y;
    int     x;
    char    *label;
} SLABEL;

static SLABEL fasLabels[] =
{
    {   0,  0, "Port:"          },
    {   0,  9, "Type:"          },
    {   0, 24, "Units:"         },
    {   0, 36, "Vec:"           },
    {   0, 46, "Sioa:"          },
    {   0, 57, "SioaOff:"       },
    {   0, 71, "Mcb:"           },
    {   1,  0, "iflag:"         },
    {   3,  0, "cflag:"         },
    {   4,  0, "device_flag:"   },
    {   5,  0, "o_state:"       },
    {   6,  0, "flow_flag:"     },
    {   7,  0, "mod_stat:"      },
    {   8,  0, "cstart:"        },
    {   8, 13, "cstop:"         },
    {   8, 25, "hupwait: "      },
    {  -1, -1, (char *) 0       }
};

typedef struct bitfld
{
    int     y;
    int     x;
    char    *label;
    int     flag_num;
    uint    mask;
} BITFLD;

#define IFLAG   1
#define DFLAG   2
#define FFLAG   3
#define EFLAG   4
#define CFLAG   5
#define OSTATE  6
#define MSTATE  7

static BITFLD ttybitflds[] =
{
    {   1,   7, "IGNBRK",   IFLAG,  IGNBRK                  },
    {   1,  14, "BRKINT",   IFLAG,  BRKINT                  },
    {   1,  21, "IGNPAR",   IFLAG,  IGNPAR                  },
    {   1,  28, "PARMRK",   IFLAG,  PARMRK                  },
    {   1,  35, "INPCK",    IFLAG,  INPCK                   },
    {   1,  41, "ISTRIP",   IFLAG,  ISTRIP                  },
    {   1,  48, "INLCR",    IFLAG,  INLCR                   },
    {   1,  54, "IGNCR",    IFLAG,  IGNCR                   },
    {   1,  60, "ICRNL",    IFLAG,  ICRNL                   },
    {   1,  66, "IUCLC",    IFLAG,  IUCLC                   },
    {   2,   7, "IXON",     IFLAG,  IXON                    },
    {   2,  14, "IXOFF",    IFLAG,  IXOFF                   },
    {   2,  21, "IXANY",    IFLAG,  IXANY                   },

    {   3,  20, "CLOCAL",   CFLAG,  CLOCAL                  },
    {   3,  27, "CREAD",    CFLAG,  CREAD                   },
    {   3,  33, "HUPCL",    CFLAG,  HUPCL                   },

    {   4,  13, "OPN",      DFLAG,  DF_DEVICE_OPEN          },
    {   4,  17, "LCK",      DFLAG,  DF_DEVICE_LOCKED        },
    {   4,  21, "ENBL",     DFLAG,  DF_MODEM_ENABLED        },
    {   4,  26, "XBSY",     DFLAG,  DF_XMIT_BUSY            },
    {   4,  31, "XBRK",     DFLAG,  DF_XMIT_BREAK           },
    {   4,  36, "XLCK",     DFLAG,  DF_XMIT_LOCKED          },
    {   4,  41, "HANG",     DFLAG,  DF_DO_HANGUP            },
    {   4,  46, "DTIM",     DFLAG,  DF_DELAY_TIMEOUT        },
    {   4,  51, "GTIM",     DFLAG,  DF_GUARD_TIMEOUT        },
    {   4,  56, "CARR",     DFLAG,  DF_CARR_ON              },
    {   4,  61, "WANT",     DFLAG,  DF_DEVICE_WANTED        },

    {   5,  13, "DLOUT",    OSTATE, OS_OPEN_FOR_DIALOUT     },
    {   5,  19, "GTTY",     OSTATE, OS_OPEN_FOR_GETTY       },
    {   5,  24, "WOPN",     OSTATE, OS_WAIT_OPEN            },
    {   5,  29, "NDIAL",    OSTATE, OS_NO_DIALOUT           },
    {   5,  35, "CCOO",     OSTATE, OS_CHECK_CARR_ON_OPEN   },
    {   5,  40, "FCO",      OSTATE, OS_FAKE_CARR_ON         },
    {   5,  44, "UNEN",     OSTATE, OS_UNBLOCK_ENABLE       },
    {   5,  49, "CL",       OSTATE, OS_CLOCAL               },
    {   5,  52, "HWHS",     OSTATE, OS_HW_HANDSHAKE         },

    {   6,  13, "HWOHS",    FFLAG,  FF_HWO_HANDSHAKE        },
    {   6,  19, "HWIHS",    FFLAG,  FF_HWI_HANDSHAKE        },
    {   6,  25, "HWOSTP",   FFLAG,  FF_HWO_STOPPED          },
    {   6,  32, "HWISTP",   FFLAG,  FF_HWI_STOPPED          },
    {   6,  39, "SWOSTP",   FFLAG,  FF_SWO_STOPPED          },
    {   6,  46, "SWISTP",   FFLAG,  FF_SWI_STOPPED          },
    {   6,  53, "SWFCREQ",  FFLAG,  FF_SW_FC_REQ            },
    {   6,  61, "RXFER",    FFLAG,  FF_RXFER_STOPPED        },
    {   6,  67, "IBLK",     FFLAG,  FF_I_BLOCKED            },

    {   7,  10, "CTS_DLT",  MSTATE, MS_CTS_DELTA            },
    {   7,  18, "DSR_DLT",  MSTATE, MS_DSR_DELTA            },
    {   7,  26, "RING_TDG", MSTATE, MS_RING_TEDGE           },
    {   7,  35, "DCD_DLT",  MSTATE, MS_DCD_DELTA            },
    {   7,  43, "CTS_PRS",  MSTATE, MS_CTS_PRESENT          },
    {   7,  51, "DSR_PRS",  MSTATE, MS_DSR_PRESENT          },
    {   7,  59, "RING_PRS", MSTATE, MS_RING_PRESENT         },
    {   7,  68, "DCD_PRS",  MSTATE, MS_DCD_PRESENT          },

    {   -1, -1, (char *)0,    -1,    -1  }
};

static char *speeds[] =
{
    "  HUP", "   50", "   75", "  110", "  134", "  150", "  200",
    "  300", "  600", " 1200", " 1800", " 2400", " 4800", " 9600",
    "19200", "38400"
};

static char *BuildDps (uint cflag)
{
    static char     rtnstr[64];
    register char   *cptr;

    if ((cflag & CBAUD) >= 0 && (cflag & CBAUD) < sizeof (speeds))
	(void) strcpy (rtnstr, speeds[cflag & CBAUD]);
    else
	(void) strcpy (rtnstr, "?????");

    cptr = rtnstr + strlen (rtnstr);
    *cptr++ = '-';

    switch (cflag & CSIZE)
    {
	case CS5: *cptr++ = '5'; break;
	case CS6: *cptr++ = '6'; break;
	case CS7: *cptr++ = '7'; break;
	case CS8: *cptr++ = '8'; break;
    }
    *cptr++ = '-';
    *cptr++ = (cflag & PARENB) ? ((cflag & PARODD) ? 'O' : 'E') : 'N';
    *cptr++ = '-';
    *cptr++ = (cflag & CSTOPB) ? '2' : '1';
    *cptr   = '\0';

    return (rtnstr);
}


static void ShowFasEntry (
    WINDOW                  *win,
    int                     port,
    int                     tly,
    int                     tlx,
    struct fas_info         *fip,
    struct fas_cont_conf    *cip
)
{
    BITFLD      *bfptr = ttybitflds;
    SLABEL      *sptr  = fasLabels;
    uint        flag;

    if (!(fip->device_flags & DF_DEVICE_CONFIGURED))
	return;

    tly++;

    use_cp (win, cpLIT);

    while (sptr->y >= 0)
    {
	wmove (win, sptr->y + tly, sptr->x + tlx);
	waddstr (win, sptr->label);
	sptr++;
    }

    use_cp (win, cpINFO);
    mvwprintw (win, tly + 0, tlx +  6, "%2d", port);
    mvwprintw (win, tly + 0, tlx + 31, "%4d", cip->units);
    mvwprintw (win, tly + 0, tlx + 41, "%4d", cip->vec);
    mvwprintw (win, tly + 0, tlx + 52, "%4x", cip->sioa);
    mvwprintw (win, tly + 0, tlx + 66, "%4x", cip->sioaoffset);
    mvwprintw (win, tly + 0, tlx + 76, "%4x", cip->mcb);

    if (fip->device_flags & DF_DEVICE_IS_NS16550)
	mvwaddstr (win, tly + 0, tlx + 15, "NS16550A");
    else if (fip->device_flags & DF_DEVICE_IS_I82510)
	mvwaddstr (win, tly + 0, tlx + 15, "  I82510");
    else
	mvwaddstr (win, tly + 0, tlx + 15, " NS16450");

    mvwaddstr (win, tly + 3, tlx + 8, BuildDps (fip->cflag));

    while (bfptr->y >= 0)
    {
	switch (bfptr->flag_num)
	{
	    case IFLAG:  flag = fip->iflag;        break;
	    case DFLAG:  flag = fip->device_flags; break;
	    case FFLAG:  flag = fip->flow_flags;   break;
	    case EFLAG:  flag = fip->event_flags;  break;
	    case CFLAG:  flag = fip->cflag;        break;
	    case OSTATE: flag = fip->o_state;      break;
	    case MSTATE: flag = fip->msr;          break;
	}

	flag &= bfptr->mask;

	wmove (win, bfptr->y + tly, bfptr->x + tlx);

	if (flag)
	    use_cp (win, cpREVERSE);

	waddstr (win, bfptr->label);

	if (flag)
	    use_cp (win, cpINFO);

	bfptr++;
    }

    mvwprintw (win, tly + 8, tlx +  8, "0x%02x", fip->cstart);
    mvwprintw (win, tly + 8, tlx + 20, "0x%02x", fip->cstop);
    mvwprintw (win, tly + 8, tlx + 34, "%-4d", fip->hupwait);
}


void FasInit (void)
{
    noSyms = GetSymbols (kmemSymbols, 0);
}


void FasCommand (int cmd)
{
    if (noSyms)
    {
	beep ();
	return;
    }

    if (cmd >= '0' && cmd <= '9')
	fasPort = cmd - '0';
    else if (cmd == ',' || cmd == KEY_UP || cmd == KEY_PREVIOUS)
	fasPort -= (fasPort > 0) ? 1 : 0;
    else if (cmd == '.' || cmd == KEY_DOWN || cmd == KEY_NEXT)
	fasPort += 1;
    else if (cmd == '<' || cmd == KEY_LEFT || cmd == KEY_BEG)
	fasPort = 0;
    else if (cmd == '>' || cmd == KEY_RIGHT || cmd == KEY_END)
	fasPort = LASTPORT;
    else
    {
	fasPort = 0;
	beep ();
    }
}


void DspFasLiterals (WINDOW *win, int tly, int tlx)
{
    static char *header = "---- FAS statistics ";
    int         x2;
    int         y;

    wmove (win, tly, tlx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x2);
    while (x2++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "FAS related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    use_cp (win, cpLIT);
    ClearArea (win, getmaxy (win) - 1, 0, getmaxx (win));
}


void DspFas (WINDOW *win, int tly, int tlx)
{
    struct fas_info         *fip;
    struct fas_cont_conf    *cip;
    int                     count;
    int                     idx;
    int                     num;
    int                     entries;

    if (noSyms)
	return;

    count = *KTUT (ADDR_fas_physical_units, int *);
    fip   = KTUT (ADDR_fas_info_data, struct fas_info *);
    cip   = KTUT (ADDR_fas_cont, struct fas_cont_conf *);

    entries = getmaxy (win) / 10;
    entries = min (count, entries);

    if (fasPort >= count)
	fasPort = count - 1;

    for (num = idx = 0; num < entries && idx < count; idx++, fip++, cip++)
    {
	if (idx < fasPort)
	    continue;

	ShowFasEntry (win, idx, tly + (10 * num), tlx, fip, cip);

	num++;
    }

    wclrtobot (win);
    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

#else
/* Prevent "empty translation unit" warnings from the idiots at X3J11. */
static char ansi_c_idiots = 69;
#endif
