/*
**
**                                H A C K . C
**
**
**      Module:     hackers variables display
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      13-OCT-1992 AV  co-proc related vars moved to hard.c
**      23-OCT-1992 AV  Version 1.0
**      06-NOV-1992 AV  conditionalized variables, don't panic if some are missing
**      03-DEC-1992 AV  added kmem stats (thanks to Juergen Krause <jkrause.bln@sni.de>)
**      21-OCT-1993 AV  UNIVEL fixes
**
*/

#include "config.h"

#if defined(MODULE_HACK)

#include <stdio.h>
#include <stdlib.h>

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  --------- Hackers Variables ----------------------------------------------------
**  lotsfree  ######k minhidustk   ####### dma_check_on  ####### Dstflag     #######
**  desfree   ######k minustkgap   ####### dma_freemem   ####### maxminor    #######
**  minfree   ######k piosegsz     ####### dma_io_slots  ####### putbufsz    #######
**  nscan     ######k syssegsz     ####### dma_limit_pfn ####### sanity_clk  #######
**  desscan   ######k segmapsz     ####### dma_single    #######
**  maxpgio   ####### piomapsz     ####### dmaable_free  ####### ncsize      #######
**  deficit   ####### piomaxsz     ####### dmaable_pages ####### filecnt     #######
**  freemem   ######k pages_pp_max ####### dmaable_sleep ####### maxclick    #######
**  pfreecnt  ####### minpagefree  ####### km_mem_small  ####### pfreecnt    #######
**  nptfree   ####### nptalloced   ####### km_alloc_small####### ngroups_max #######
**  availrmem ######k npgoutbuf    ####### km_mem_big    ####### rstchown    #######
**  availkmem ######k nnotpgoutbuf ####### km_alloc_big  ####### Hz          #######
**  availsmem ######k exec_ncargs  ####### km_alloc_ovsz ####### Timezone    #######
**
**  Should add theese
**
**  size    #pools  in_pool    alloc #fail
**  small     #### ######## ########  ####
**  big       #### ######## ########  ####
**  outsize   #### ######## ########  ####
**
*/

#define ADDR_Dstflag            kmemSymbols[ 0].n_value
#define ADDR_Hz                 kmemSymbols[ 1].n_value
#define ADDR_Timezone           kmemSymbols[ 2].n_value
#define ADDR_availkmem          kmemSymbols[ 3].n_value
#define ADDR_availrmem          kmemSymbols[ 4].n_value
#define ADDR_availsmem          kmemSymbols[ 5].n_value
#define ADDR_deficit            kmemSymbols[ 6].n_value
#define ADDR_desfree            kmemSymbols[ 7].n_value
#define ADDR_desscan            kmemSymbols[ 8].n_value
#define ADDR_exec_ncargs        kmemSymbols[ 9].n_value
#define ADDR_filecnt            kmemSymbols[10].n_value
#define ADDR_freemem            kmemSymbols[11].n_value
#define ADDR_lotsfree           kmemSymbols[12].n_value
#define ADDR_maxclick           kmemSymbols[13].n_value
#define ADDR_maxminor           kmemSymbols[14].n_value
#define ADDR_maxpgio            kmemSymbols[15].n_value
#define ADDR_minfree            kmemSymbols[16].n_value
#define ADDR_minhidustk         kmemSymbols[17].n_value
#define ADDR_minpagefree        kmemSymbols[18].n_value
#define ADDR_minustkgap         kmemSymbols[19].n_value
#define ADDR_ncsize             kmemSymbols[20].n_value
#define ADDR_ngroups_max        kmemSymbols[21].n_value
#define ADDR_nnotpgoutbuf       kmemSymbols[22].n_value
#define ADDR_npgoutbuf          kmemSymbols[23].n_value
#define ADDR_nptalloced         kmemSymbols[24].n_value
#define ADDR_nptfree            kmemSymbols[25].n_value
#define ADDR_nscan              kmemSymbols[26].n_value
#define ADDR_pages_pp_maximum   kmemSymbols[27].n_value
#define ADDR_pfreecnt           kmemSymbols[28].n_value
#define ADDR_piomapsz           kmemSymbols[29].n_value
#define ADDR_piomaxsz           kmemSymbols[30].n_value
#define ADDR_piosegsz           kmemSymbols[31].n_value
#define ADDR_putbufsz           kmemSymbols[32].n_value
#define ADDR_rstchown           kmemSymbols[33].n_value
#define ADDR_sanity_clk         kmemSymbols[34].n_value
#define ADDR_segmapsz           kmemSymbols[35].n_value
#define ADDR_syssegsz           kmemSymbols[36].n_value
#define ADDR_dma_check_on       kmemSymbols[37].n_value
#define ADDR_dma_freemem        kmemSymbols[38].n_value
#define ADDR_dma_io_slots       kmemSymbols[39].n_value
#define ADDR_dma_limit_pfn      kmemSymbols[40].n_value
#define ADDR_dma_single         kmemSymbols[41].n_value
#define ADDR_dmaable_free       kmemSymbols[42].n_value
#define ADDR_dmaable_pages      kmemSymbols[43].n_value
#define ADDR_dmaable_sleep      kmemSymbols[44].n_value
#define ADDR_kmeminfo           kmemSymbols[45].n_value

static struct nlist kmemSymbols[] =
{
    { "Dstflag"             },
    { "Hz"                  },
    { "Timezone"            },
    { "availkmem"           },
    { "availrmem"           },
    { "availsmem"           },
    { "deficit"             },
    { "desfree"             },
    { "desscan"             },
    { "exec_ncargs"         },
    { "filecnt"             },
#ifdef S_UNIVEL
    { "mem_freemem"         },
#else
    { "freemem"             },
#endif
    { "lotsfree"            },
    { "maxclick"            },
    { "maxminor"            },
    { "maxpgio"             },
    { "minfree"             },
    { "minhidustk"          },
    { "minpagefree"         },
    { "minustkgap"          },
    { "ncsize"              },
    { "ngroups_max"         },
    { "nnotpgoutbuf"        },
    { "npgoutbuf"           },
    { "nptalloced"          },
    { "nptfree"             },
    { "nscan"               },
    { "pages_pp_maximum"    },
    { "pfreecnt"            },
    { "piomapsz"            },
    { "piomaxsz"            },
    { "piosegsz"            },
    { "putbufsz"            },
    { "rstchown"            },
    { "sanity_clk"          },
    { "segmapsz"            },
    { "syssegsz"            },
    { "dma_check_on"        },
    { "dma_freemem"         },
    { "dma_io_slots"        },
    { "dma_limit_pfn"       },
    { "dma_single"          },
    { "dmaable_free"        },
    { "dmaable_pages"       },
    { "dmaable_sleep"       },
    { "kmeminfo"            },
    { 0                     },
};


DspHackLiterals (WINDOW *win, int ty, int tx)
{
    static char *header = "--------- Hackers Variables ";
    int         y;
    int         x;

    wmove (win, ty, tx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    use_cp (win, cpLIT);

    y = ty + 1; x = tx;
    mvwaddstr (win, y++, x, "lotsfree");
    mvwaddstr (win, y++, x, "desfree");
    mvwaddstr (win, y++, x, "minfree");
    mvwaddstr (win, y++, x, "nscan");
    mvwaddstr (win, y++, x, "desscan");
    mvwaddstr (win, y++, x, "maxpgio");
    mvwaddstr (win, y++, x, "deficit");
    mvwaddstr (win, y++, x, "freemem");
    mvwaddstr (win, y++, x, "pfreecnt");
    mvwaddstr (win, y++, x, "nptfree");
    mvwaddstr (win, y++, x, "availrmem");
    mvwaddstr (win, y++, x, "availkmem");
    mvwaddstr (win, y++, x, "availsmem");

    y = ty + 1; x = tx + 18;
    mvwaddstr (win, y++, x, "minhidustk");
    mvwaddstr (win, y++, x, "minustkgap");
    mvwaddstr (win, y++, x, "piosegsz");
    mvwaddstr (win, y++, x, "syssegsz");
    mvwaddstr (win, y++, x, "segmapsz");
    mvwaddstr (win, y++, x, "piomapsz");
    mvwaddstr (win, y++, x, "piomaxsz");
    mvwaddstr (win, y++, x, "pages_pp_max");
    mvwaddstr (win, y++, x, "minpagefree");
    mvwaddstr (win, y++, x, "nptalloced");
    mvwaddstr (win, y++, x, "npgoutbuf");
    mvwaddstr (win, y++, x, "nnotpgoutbuf");
    mvwaddstr (win, y++, x, "exec_ncargs");

    y = ty + 1; x = tx + 39;
    mvwaddstr (win, y++, x, "dma_check_on");
    mvwaddstr (win, y++, x, "dma_freemem");
    mvwaddstr (win, y++, x, "dma_io_slots");
    mvwaddstr (win, y++, x, "dma_limit_pfn");
    mvwaddstr (win, y++, x, "dma_single");
    mvwaddstr (win, y++, x, "dmaable_free");
    mvwaddstr (win, y++, x, "dmaable_pages");
    mvwaddstr (win, y++, x, "dmaable_sleep");
    mvwaddstr (win, y++, x, "km_mem_small");
    mvwaddstr (win, y++, x, "km_alloc_small");
    mvwaddstr (win, y++, x, "km_mem_big");
    mvwaddstr (win, y++, x, "km_alloc_big");
    mvwaddstr (win, y++, x, "km_alloc_ovsz");

    y = ty + 1; x = 61;
    mvwaddstr (win, y++, x, "Dstflag");
    mvwaddstr (win, y++, x, "maxminor");
    mvwaddstr (win, y++, x, "putbufsz");
    mvwaddstr (win, y++, x, "sanity_clk");
    y++;
    mvwaddstr (win, y++, x, "ncsize");
    mvwaddstr (win, y++, x, "filecnt");
    mvwaddstr (win, y++, x, "maxclick");
    mvwaddstr (win, y++, x, "pfreecnt");
    mvwaddstr (win, y++, x, "ngroups_max");
    mvwaddstr (win, y++, x, "rstchown");
    mvwaddstr (win, y++, x, "Hz");
    mvwaddstr (win, y++, x, "Timezone");
}


void DspHack (WINDOW *win, int ty, int tx)
{
    int     y;
    int     x;
    int     fpkind;

    use_cp (win, cpINFO);

    y = ty + 1; x = tx + 10;
    if (ADDR_lotsfree != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_lotsfree,  int *)));
    else
	y++;
    if (ADDR_desfree != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_desfree,   int *)));
    else
	y++;
    if (ADDR_minfree != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_minfree,   int *)));
    else
	y++;
    if (ADDR_nscan != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_nscan,     int *)));
    else
	y++;
    if (ADDR_desscan != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_desscan,   int *)));
    else
	y++;
    if (ADDR_maxpgio != 0)
	mvwprintw (win, y++, x, "%7d",           *KTUT (ADDR_maxpgio,   int *));
    else
	y++;
    if (ADDR_deficit != 0)
	mvwprintw (win, y++, x, "%7d",           *KTUT (ADDR_deficit,   int *));
    else
	y++;
    if (ADDR_freemem != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_freemem,   int *)));
    else
	y++;
    if (ADDR_pfreecnt != 0)
	mvwprintw (win, y++, x, "%7d",           *KTUT (ADDR_pfreecnt,  int *));
    else
	y++;
    if (ADDR_nptfree != 0)
	mvwprintw (win, y++, x, "%7d",           *KTUT (ADDR_nptfree,   int *));
    else
	y++;
    if (ADDR_availrmem != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_availrmem, int *)));
    else
	y++;
    if (ADDR_availkmem != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_availkmem, int *)));
    else
	y++;
    if (ADDR_availsmem != 0)
	mvwprintw (win, y++, x, "%6dk", pagetok (*KTUT (ADDR_availsmem, int *)));
    else
	y++;

    y = ty + 1; x = tx + 31;
    if (ADDR_minhidustk != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_minhidustk,       int *));
    else
	y++;
    if (ADDR_minustkgap != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_minustkgap,       int *));
    else
	y++;
    if (ADDR_piosegsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_piosegsz,         int *));
    else
	y++;
    if (ADDR_syssegsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_syssegsz,         int *));
    else
	y++;
    if (ADDR_segmapsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_segmapsz,         int *));
    else
	y++;
    if (ADDR_piomapsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_piomapsz,         int *));
    else
	y++;
    if (ADDR_piomaxsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_piomaxsz,         int *));
    else
	y++;
    if (ADDR_pages_pp_maximum != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_pages_pp_maximum, unsigned int *));
    else
	y++;
    if (ADDR_minpagefree != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_minpagefree,      int *));
    else
	y++;
    if (ADDR_nptalloced != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_nptalloced,       int *));
    else
	y++;
    if (ADDR_npgoutbuf != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_npgoutbuf,        int *));
    else
	y++;
    if (ADDR_nnotpgoutbuf != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_nnotpgoutbuf,     int *));
    else
	y++;
    if (ADDR_exec_ncargs != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_exec_ncargs,      int *));
    else
	y++;

    y = ty + 1; x = tx + 53;
    if (ADDR_dma_check_on != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_dma_check_on,  int *));
    else
	y++;
    if (ADDR_dma_freemem != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_dma_freemem,   unsigned int *));
    else
	y++;
    if (ADDR_dma_io_slots != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_dma_io_slots,  unsigned int *));
    else
	y++;
    if (ADDR_dma_limit_pfn != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_dma_limit_pfn, int *));
    else
	y++;
    if (ADDR_dma_single != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_dma_single,    int *));
    else
	y++;
    if (ADDR_dmaable_free != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_dmaable_free,  unsigned int *));
    else
	y++;
    if (ADDR_dmaable_pages != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_dmaable_pages, unsigned int *));
    else
	y++;
    if (ADDR_dmaable_sleep != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_dmaable_sleep, int *));
    else
	y++;

#define SM_MEM      0
#define BM_MEM      4
#define OM_MEM      8

#define SM_ALLOC    12
#define BM_ALLOC    16
#define OM_ALLOC    20

#define SM_FAIL     24
#define BM_FAIL     28
#define OM_FAIL     32

    if (ADDR_kmeminfo != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_kmeminfo + SM_MEM, ulong *));
    else
	y++;
    if (ADDR_kmeminfo != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_kmeminfo + SM_ALLOC, ulong *));
    else
	y++;
    if (ADDR_kmeminfo != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_kmeminfo + BM_MEM, ulong *));
    else
	y++;
    if (ADDR_kmeminfo != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_kmeminfo + BM_ALLOC, ulong *));
    else
	y++;
    if (ADDR_kmeminfo != 0)
	mvwprintw (win, y++, x, "%7u", *KTUT (ADDR_kmeminfo + OM_ALLOC, ulong *));
    else
	y++;

    y = ty + 1; x = tx + 73;
    if (ADDR_Dstflag != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_Dstflag,     int *));
    else
	y++;
    if (ADDR_maxminor != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_maxminor,    int *));
    else
	y++;
    if (ADDR_putbufsz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_putbufsz,    int *));
    else
	y++;
    if (ADDR_sanity_clk != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_sanity_clk,  int *));
    else
	y++;
    y++;
    if (ADDR_ncsize != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_ncsize,      int *));
    else
	y++;
    if (ADDR_filecnt != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_filecnt,     int *));
    else
	y++;
    if (ADDR_maxclick != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_maxclick,    int *));
    else
	y++;
    if (ADDR_pfreecnt != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_pfreecnt,    int *));
    else
	y++;
    if (ADDR_ngroups_max != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_ngroups_max, int *));
    else
	y++;
    if (ADDR_rstchown != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_rstchown,    int *));
    else
	y++;
    if (ADDR_Hz != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_Hz,          int *));
    else
	y++;
    if (ADDR_Timezone != 0)
	mvwprintw (win, y++, x, "%7d", *KTUT (ADDR_Timezone,    int *));
    else
	y++;

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

void HackInit ()
{
    (void) GetSymbols (kmemSymbols, 0);
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspHackLiterals (WINDOW *win, int tly, int tlx)
{
    char    *message = "Hacker's (misc) info not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void DspHack (WINDOW *win, int tly, int tlx)
{
}

void HackInit ()
{
}

#endif
