/*
**
**                                H E L P . C
**
**
**      Module:     help display
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      17-OCT-1992 AV  Version X1.0, first definition
**      23-OCT-1992 AV  Version 1.0
**      25-OCT-1992 AV  add SLIP entry
**      21-OCT-1993 AV  minor change in help page
**      12-NOV-1993 AV  added entry for '|' (toggle logging)
**
*/

#include "config.h"

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

static int  helpPage = 0;

#define LASTPAGE    9999

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  ---- HELP ----------------------------------------------------------------------
**  q   quit program                        A   absolute mode (sys/vm-info)
**  l   lock process into memory (if root)  D   delta mode (sys/vm-info)
**  u   unlock process                      R   relative mode (sys/vm-info)
**  p   short process listing               0-9 port number (serial)
**  P   long process listing (if LINES>24)  @   toggle short/long command (proc)
**  e   extra display (if LINES=24)         0-9 page number (proc)
**  s   STREAMS display                     ,   minus page (proc)
**  a   architecture and devices display    .   plus page (proc)
**  t   TCP display                         <   first page (proc)
**  i   IP display                          >   last page (proc)
**  x   serial driver display
**  c   SCSI devices display                %   toggle flags/performance (scsi)
**  z   SLIP display
**  ?   show this help page                 h   misc (hackers) variables
*/

void DspHelpLiterals (WINDOW *win, int ty, int tx)
{
    static char *header = "---- HELP ";
    int     y;
    int     x;

    use_cp (win, cpBANNER);
    mvwaddstr (win, 0, 0, header);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');
}


/*
**   00000000001111111111222222222233333333334444444444555555555566666666667777777777
**   01234567890123456789012345678901234567890123456789012345678901234567890123456789
*/
static char *page1[] = {
    "svr4mon version 1.3, Copyright (C) 1992,1993 Andreas Vogel",
    "This is free software; see the source for copying conditions.",
    "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A",
    "PARTICULAR PURPOSE.",
    "",
    "General Commands",
    "",
    "q       quit program                         CTRL-L,CTRL-R  refresh screen",
    "l       lock process into memory (if root)   u        unlock process",
    "L       change to RT sched. class (if root)  U        change to TS sched. class",
    "?       show this help page",
    "",
    ", UP    previous help page                   . DOWN   next help page",
    "< LEFT  first help page                      > RIGHT  last help page",
    0
};

static char *page2[] = {
    "p   short process listing",
    "P   long process listing (if LINES > 38)",
    "@   toggle short/long command display (proc)",
    "m   sysinfo/vminfo",
    "h   misc (hackers) variables",
    "e   extra display",
    "s   STREAMS display",
    "a   architecture and devices display",
    "t   TCP display",
    "i   IP display",
    "x   serial driver display",
    "z   SLIP display",
    "m   main screen",
    "c   SCSI devices display",
    "%   toggle flags/performance (SCSI)",
    0
};

static char *page3[] = {
    "Modes:",
    "",
    "A   absolute mode (where applicable)",
    "D   delta mode (where applicable)",
    "R   relative mode (where applicable)",
    "",
    "|   toggle logging to file (only if invoked with -o and -C)",
    0
};

static char **pages[] = { page1, page2, page3 };

#define NumPages    sizeof (pages) / sizeof (pages[0])


void DspHelp (WINDOW *win, int ty, int tx)
{
    int     lastRow = getmaxy (win) - 1;
    int     x;
    int     y;
    char    **ptr;

    if (helpPage < 0)
	helpPage = 0;

    if (helpPage >= NumPages || helpPage == LASTPAGE)
	helpPage = NumPages - 1;


    use_cp (win, cpBANNER);
    mvwprintw (win, 0, 18, " Page %d of %d ", helpPage, NumPages - 1);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    use_cp (win, cpLIT);

    ptr = pages[helpPage];
    ty++;
    while (*ptr != (char *) 0 && ty <= lastRow)
    {
	mvwaddstr (win, ty++, tx, *ptr++);
	getyx (win, y, x);
	while (x++ < getmaxx (win))
	    waddch (win, (chtype) ' ');
    }

    wclrtobot (win);
    wmove (win, lastRow, getmaxx (win) - 1);
}


void HelpCommand (int cmd)
{
    if (cmd >= '0' && cmd <= '9')
	helpPage = cmd - '0';
    else if (cmd == ',' || cmd == KEY_UP || cmd == KEY_PREVIOUS)
	helpPage -= (helpPage > 0) ? 1 : 0;
    else if (cmd == '.' || cmd == KEY_DOWN || cmd == KEY_NEXT)
	helpPage += 1;
    else if (cmd == '<' || cmd == KEY_LEFT || cmd == KEY_BEG)
	helpPage = 0;
    else if (cmd == '>' || cmd == KEY_RIGHT || cmd == KEY_END)
	helpPage = LASTPAGE;
    else
    {
	helpPage = 0;
	beep ();
    }
}



void HelpInit ()
{
}
