/*
**
**                              P A G I N G . H
**
**
**      Module:     header file for paging and symbol routines
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      15-OCT-1992 AV  fas and sas entries were not correct
**      16-OCT-1992 AV  added tcp,ip,udp,icmp stat variables
**      16-OCT-1992 AV  even some more kernel vars
**      17-OCT-1992 AV  use SYM_DUMMY for unavailble kernel vars
**                      pad fas and sas structures
**      20-OCT-1992 AV  asy support
**      21-OCT-1992 AV  start port for ARCH_MOT_88k
**      22-OCT-1992 AV  just an important improvement (maybe fix):
**                      in the past, we've used KVBASE as the base address
**                      for all kernel page mappings. This changed now.
**                      On some systems, some (or all) pages in the range from
**                      KVBASE to KVSBASE aren't materialized and as a
**                      consequence the mmap() failed. Anyway, we only need
**                      access to kernel pages in the range [KVSBASE,UVUBLK]
**                      which holds the kernel text, data and bss sections.
**      23-OCT-1992 AV  Version 1.0
**      10-NOV-1992 AV  get kernel min and max address from /unix
**
*/

#ifndef _paging_h_
#define _paging_h_

#include <nlist.h>

#if defined(S_MOTOROLA_88K)
#   include <sys/sbd.h>
#else
#   include <sys/immu.h>
#endif

/*
**  min/max kernel virtual address, see paging.c
*/
#define KERNEL_VS_MIN       minkvadr
#define KERNEL_VS_MAX       maxkvadr

#include <sys/sysmacros.h>

/*
**  several special devices
*/
#define VMUNIX_FILE     "/stand/unix"
#define KMEM_FILE       "/dev/kmem"
#define MEM_FILE        "/dev/mem"


#define __C(x)      ((caddr_t) x)

/*
**  kernel address to user address
*/
#define __KTU(k)    \
    ( __C(k) < __C(KERNEL_VS_MIN) || __C(k) > __C(KERNEL_VS_MAX) ? 0 : (__C(k) - __C(KERNEL_VS_MIN))   + __C(pagingBase) )

/*
**  user address to kernel address
*/
#define __UTK(u)    \
    ( __C(u) < __C(pagingBase) ? 0 : (__C(u) - __C(pagingBase)) + __C(KERNEL_VS_MIN) )

/*
**  kernel address to user address with type cast
*/
#define KTUT(k,type)    ((type) __KTU(k))

/*
**  user address to kernel address with type cast
*/
#define UTKT(u,type)    ((type) __UTK(u))

#define KTU(k)          KTUT(k, caddr_t)
#define UTK(u)          UTKT(u, caddr_t)

#define LOG1024             10
#define FSCALE              (1 << 8)

#define percent_cpu(cpu)    ((double) ((double) (cpu)) / ((double) FSCALE))
#define pagetok(size)       (ctob (size) >> LOG1024)
#define bytestok(size)      ((size) >> LOG1024)
#define ktobytes(size)      ((size) << LOG1024)
#define bytestopage(size)   btoc (size)

/*
**  dummy entry for nlist array entries not currently used
*/
#define SYM_DUMMY       "_dummy_"

/*
**  base address of mapped kernel pages in user space
**  starting at this base address, all pages must be mapped contiguous!!
*/
extern caddr_t          pagingBase;
extern caddr_t          minkvadr;
extern caddr_t          maxkvadr;

extern int      GetSymbols (struct nlist *list, int check);
extern void     SetupPaging (void);

#endif

