/*
**
**                                 S A S . C
**
**
**      Module:     support for sas
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      10-OCT-1992 AV  Version X1.0, first definition
**      15-OCT-1992 AV  fixed some bugs reported by dwex@mtgzfs3.att.com
**      23-OCT-1992 AV  Version 1.0
**      30-NOV-1992 AV  corrected speed display for HIGH_SPEED mode
**                      thanks to nms@ussr.EU.net (Nickolay Saukh)
**
*/

#include "config.h"

#if defined(MODULE_SAS)

#include <stdio.h>
#include <stdlib.h>
#include <curses.h>
#include <panel.h>
#include <termios.h>

/*
**  NOTE: Check carefully this two defines. They must be set to the same
**        value as for the kernel build. Just define the same symbols
**        that were defined for kernel compilation. Otherwise structure
**        sizes differs and invalid information will retrieved.
*/
#define MERGE386
#undef  HAVE_VPIX
#include <sys/sas.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  Port: xx Type: xxxxxxxx Vec: xxxx Sioa: xxxx Sioaoffset: xxxx Mcb: xxxx
**  iflag: BRKINT DOSMODE ICRNL IGNBRK IGNCR IGNPAR IMAXBEL INLCR INPCK ISTRIP
**         IUCLC  IXANY   IXOFF IXON   PARMRK
**  cflag:  38400-8-N-1 CLOCAL CREAD HUPCL
**  device_flag: CTLF CTLE OPN LCK ENBL XBSY XBRK XLCK HANG DBRK GTIM FDM MSI HIGHS
**  o_state:     CLOS DLOUT GTTY WOPN NDIAL FCO CL HWOH HWIH HDXH EXO1 EXO2
**  flow_flag:  HWOHS HWIHS HDXHS HWOSTP HWISTP HDXSTA SWOSTP SWISTP SWFCREQ RXFRSTP
**  mod_stat: CTS_DLT DSR_DLT RING_TDG DCD_DLT CTS_PRS DSR_PRS RING_PRS DCD_PRS
**  start xx stop xx intr xx quit xx erase xx kill xx eof xx eol xx vmin xx vtime xx
**
*/

#define ADDR_sas_ctl_port           kmemSymbols[ 0].n_value
#define ADDR_sas_ctl_val            kmemSymbols[ 1].n_value
#define ADDR_sas_flow               kmemSymbols[ 2].n_value
#define ADDR_sas_info               kmemSymbols[ 3].n_value
#define ADDR_sas_info_ptr           kmemSymbols[ 4].n_value
#define ADDR_sas_init_seq           kmemSymbols[ 5].n_value
#define ADDR_sas_int_ack            kmemSymbols[ 6].n_value
#define ADDR_sas_int_ack_port       kmemSymbols[ 7].n_value
#define ADDR_sas_mcb                kmemSymbols[ 8].n_value
#define ADDR_sas_modem              kmemSymbols[ 9].n_value
#define ADDR_sas_physical_units     kmemSymbols[10].n_value
#define ADDR_sas_port               kmemSymbols[11].n_value
#define ADDR_sas_vec                kmemSymbols[12].n_value

static struct nlist kmemSymbols[] =
{
    { "sas_ctl_port"        },
    { "sas_ctl_val"         },
    { "sas_flow"            },
    { "sas_info"            },
    { "sas_info_ptr"        },
    { "sas_init_seq"        },
    { "sas_int_ack"         },
    { "sas_int_ack_port"    },
    { "sas_mcb"             },
    { "sas_modem"           },
    { "sas_physical_units"  },
    { "sas_port"            },
    { "sas_vec"             },
    { 0                     },
};

static int      sasPort = 0;
static int      noSyms;

#define LASTPORT    9999

typedef struct slabel
{
    int     y;
    int     x;
    char    *label;
} SLABEL;

static SLABEL sasLabels[] =
{
    {   0,  0, "Port:"          },
    {   0,  9, "Type:"          },
    {   0, 24, "Vec:"           },
    {   0, 34, "Sioa:"          },
    {   0, 45, "SioaOffset:"    },
    {   0, 62, "Mcb:"           },
    {   1,  0, "iflag:"         },
    {   3,  0, "cflag:"         },
    {   4,  0, "device_flag:"   },
    {   5,  0, "o_state:"       },
    {   6,  0, "flow_flag:"     },
    {   7,  0, "mod_stat:"      },
    {   8,  0, "start"          },
    {   8,  9, "stop"           },
    {   8, 17, "intr"           },
    {   8, 25, "quit"           },
    {   8, 33, "erase"          },
    {   8, 42, "kill"           },
    {   8, 50, "eof"            },
    {   8, 57, "eol"            },
    {   8, 64, "vmin"           },
    {   8, 72, "vtime"          },
    {  -1, -1, (char *) 0       }
};

typedef struct bitfld
{
    int     y;
    int     x;
    char    *label;
    int     flag_num;
    uint    mask;
} BITFLD;

#define IFLAG   1
#define DFLAG   2
#define FFLAG   3
#define EFLAG   4
#define CFLAG   5
#define OSTATE  6
#define MSTATE  7

static BITFLD ttybitflds[] =
{
    {   1,   7, "IGNBRK",   IFLAG,  IGNBRK                  },
    {   1,  14, "BRKINT",   IFLAG,  BRKINT                  },
    {   1,  21, "IGNPAR",   IFLAG,  IGNPAR                  },
    {   1,  28, "PARMRK",   IFLAG,  PARMRK                  },
    {   1,  35, "INPCK",    IFLAG,  INPCK                   },
    {   1,  41, "ISTRIP",   IFLAG,  ISTRIP                  },
    {   1,  48, "INLCR",    IFLAG,  INLCR                   },
    {   1,  54, "IGNCR",    IFLAG,  IGNCR                   },
    {   1,  60, "ICRNL",    IFLAG,  ICRNL                   },
    {   1,  66, "IUCLC",    IFLAG,  IUCLC                   },
    {   2,   7, "IXON",     IFLAG,  IXON                    },
    {   2,  14, "IXOFF",    IFLAG,  IXOFF                   },
    {   2,  21, "IXANY",    IFLAG,  IXANY                   },

    {   3,  20, "CLOCAL",   CFLAG,  CLOCAL                  },
    {   3,  27, "CREAD",    CFLAG,  CREAD                   },
    {   3,  33, "HUPCL",    CFLAG,  HUPCL                   },

    {   4,  13, "CTLF",     DFLAG,  DF_CTL_FIRST            },
    {   4,  18, "CTLE",     DFLAG,  DF_CTL_EVERY            },
    {   4,  23, "OPN",      DFLAG,  DF_DEVICE_OPEN          },
    {   4,  27, "LCK",      DFLAG,  DF_DEVICE_LOCKED        },
    {   4,  31, "ENBL",     DFLAG,  DF_MODEM_ENABLED        },
    {   4,  36, "XBSY",     DFLAG,  DF_XMIT_BUSY            },
    {   4,  41, "XBRK",     DFLAG,  DF_XMIT_BREAK           },
    {   4,  46, "XLCK",     DFLAG,  DF_XMIT_LOCKED          },
    {   4,  51, "HANG",     DFLAG,  DF_DO_HANGUP            },
    {   4,  56, "DBRK",     DFLAG,  DF_DO_BREAK             },
    {   4,  61, "GTIM",     DFLAG,  DF_GUARD_TIMEOUT        },
    {   4,  66, "FDM",      DFLAG,  DF_FIFO_DROP_MODE       },
    {   4,  70, "MSI",      DFLAG,  DF_MSI_ENABLED          },
    {   4,  74, "HIGHS",    DFLAG,  DF_HIGH_SPEED           },

    {   5,  18, "DLOUT",    OSTATE, OS_OPEN_FOR_DIALOUT     },
    {   5,  24, "GTTY",     OSTATE, OS_OPEN_FOR_GETTY       },
    {   5,  29, "WOPN",     OSTATE, OS_WAIT_OPEN            },
    {   5,  34, "NDIAL",    OSTATE, OS_NO_DIALOUT           },
    {   5,  40, "FCO",      OSTATE, OS_FAKE_CARR_ON         },
    {   5,  44, "CL",       OSTATE, OS_CLOCAL               },
    {   5,  47, "HWOH",     OSTATE, OS_HWO_HANDSHAKE        },
    {   5,  52, "HWIH",     OSTATE, OS_HWI_HANDSHAKE        },
    {   5,  57, "HDXH",     OSTATE, OS_HDX_HANDSHAKE        },
    {   5,  62, "EXO1",     OSTATE, OS_EXCLUSIVE_OPEN_1     },
    {   5,  67, "EXO2",     OSTATE, OS_EXCLUSIVE_OPEN_2     },

    {   6,  12, "HWOHS",    FFLAG,  FF_HWO_HANDSHAKE        },
    {   6,  18, "HWIHS",    FFLAG,  FF_HWI_HANDSHAKE        },
    {   6,  24, "HDXHS",    FFLAG,  FF_HDX_HANDSHAKE        },
    {   6,  30, "HWOSTP",   FFLAG,  FF_HWO_STOPPED          },
    {   6,  37, "HWISTP",   FFLAG,  FF_HWI_STOPPED          },
    {   6,  44, "HDXSTA",   FFLAG,  FF_HDX_STARTED          },
    {   6,  51, "SWOSTP",   FFLAG,  FF_SWO_STOPPED          },
    {   6,  58, "SWISTP",   FFLAG,  FF_SWI_STOPPED          },
    {   6,  65, "SWFCREQ",  FFLAG,  FF_SW_FC_REQ            },
    {   6,  73, "RXFRSTP",  FFLAG,  FF_RXFER_STOPPED        },

    {   7,  10, "CTS_DLT",  MSTATE, MS_CTS_DELTA            },
    {   7,  18, "DSR_DLT",  MSTATE, MS_DSR_DELTA            },
    {   7,  26, "RING_TDG", MSTATE, MS_RING_TEDGE           },
    {   7,  35, "DCD_DLT",  MSTATE, MS_DCD_DELTA            },
    {   7,  43, "CTS_PRS",  MSTATE, MS_CTS_PRESENT          },
    {   7,  51, "DSR_PRS",  MSTATE, MS_DSR_PRESENT          },
    {   7,  59, "RING_PRS", MSTATE, MS_RING_PRESENT         },
    {   7,  68, "DCD_PRS",  MSTATE, MS_DCD_PRESENT          },

    {  -1,  -1, (char *) 0, -1,     -1                      }
};

static char *speeds[] =
{
    "   HUP", "    50", "    75", "   110", "   134", "   150", "   200",
    "   300", "   600", "  1200", "  1800", "  2400", "  4800", "  9600",
    " 19200", " 38400"
};

static char *h_speeds[] =
{
    " 57600", "    50", "    75", "   110", "   134", "   150", "   200",
    "   300", "   600", "  1200", "115200", "  2400", "  4800", "  9600",
    " 19200", " 38400"
};

static char *BuildDps (uint cflag, int hspd)
{
    static char     rtnstr[32];
    register char   *cptr;

    if ((cflag & CBAUD) >= 0 && (cflag & CBAUD) < sizeof (speeds))
	(void) strcpy (
	    rtnstr,
	    hspd ? h_speeds[cflag & CBAUD] : speeds[cflag & CBAUD]
	);
    else
	(void) strcpy (rtnstr, "?????");

    cptr = rtnstr + strlen (rtnstr);
    *cptr++ = '-';

    switch (cflag & CSIZE)
    {
	case CS5: *cptr++ = '5'; break;
	case CS6: *cptr++ = '6'; break;
	case CS7: *cptr++ = '7'; break;
	case CS8: *cptr++ = '8'; break;
    }
    *cptr++ = '-';
    *cptr++ = (cflag & PARENB) ? ((cflag & PARODD) ? 'O' : 'E') : 'N';
    *cptr++ = '-';
    *cptr++ = (cflag & CSTOPB) ? '2' : '1';
    *cptr   = '\0';

    return (rtnstr);
}


static void ShowSasEntry (
    WINDOW              *win,
    int                 port,
    int                 tly,
    int                 tlx,
    struct sas_info     *sip,
    ulong               sas_port,
    uint                sas_vec,
    uint                sas_mcb
)
{
    BITFLD      *bfptr = ttybitflds;
    SLABEL      *sptr  = sasLabels;
    uint        flag;

    if (!(sip->device_flags.i & DF_DEVICE_CONFIGURED))
	return;

    tly++;

    use_cp (win, cpLIT);

    while (sptr->y >= 0)
    {
	wmove (win, sptr->y + tly, sptr->x + tlx);
	waddstr (win, sptr->label);
	sptr++;
    }

    use_cp (win, cpINFO);

    mvwprintw (win, tly + 0, tlx +  6, "%2d", port);
    mvwprintw (win, tly + 0, tlx + 29, "%4d", sas_vec);
    mvwprintw (win, tly + 0, tlx + 40, "%4x", sas_port & 0xffff);
    mvwprintw (win, tly + 0, tlx + 57, "%4x", 0);
    mvwprintw (win, tly + 0, tlx + 67, "%4x", sas_mcb);

    while (bfptr->y >= 0)
    {
	switch (bfptr->flag_num)
	{
	    case IFLAG:  flag = sip->iflag;          break;
	    case DFLAG:  flag = sip->device_flags.i; break;
	    case FFLAG:  flag = sip->flow_flags.i;   break;
	    case EFLAG:  flag = sip->event_flags.i;  break;
	    case CFLAG:  flag = sip->cflag;          break;
	    case OSTATE: flag = sip->o_state;        break;
	    case MSTATE: flag = sip->msr;            break;
	}

	flag &= bfptr->mask;

	wmove (win, bfptr->y + tly, bfptr->x + tlx);

	if (flag)
	    use_cp (win, cpREVERSE);

	waddstr (win, bfptr->label);

	if (flag)
	    use_cp (win, cpINFO);

	bfptr++;
    }

    if (sip->device_type == TYPE_NS16450)
	mvwaddstr (win, tly + 0, tlx + 15, " NS16450");
    else if (sip->device_type == TYPE_I82510)
	mvwaddstr (win, tly + 0, tlx + 15, "  I82510");
    else if (sip->device_type == TYPE_NS16550A)
	mvwaddstr (win, tly + 0, tlx + 15, "NS16550A");
    else
	mvwaddstr (win, tly + 0, tlx + 15, " UNKNOWN");

    mvwaddstr (win, tly + 3, tlx + 7,
	BuildDps (sip->cflag, sip->device_flags.i & DF_HIGH_SPEED)
    );

    mvwprintw (win, tly + 8, tlx +  6, "%02x", sip->cc[VSTART]);
    mvwprintw (win, tly + 8, tlx + 14, "%02x", sip->cc[VSTOP]);
    mvwprintw (win, tly + 8, tlx + 22, "%02x", sip->cc[VINTR]);
    mvwprintw (win, tly + 8, tlx + 30, "%02x", sip->cc[VQUIT]);
    mvwprintw (win, tly + 8, tlx + 39, "%02x", sip->cc[VERASE]);
    mvwprintw (win, tly + 8, tlx + 47, "%02x", sip->cc[VKILL]);
    mvwprintw (win, tly + 8, tlx + 54, "%02x", sip->cc[VEOF]);
    mvwprintw (win, tly + 8, tlx + 61, "%02x", sip->cc[VEOL]);
    mvwprintw (win, tly + 8, tlx + 69, "%02x", sip->cc[VMIN]);
    mvwprintw (win, tly + 8, tlx + 78, "%02x", sip->cc[VTIME]);
}


void SasInit (void)
{
    noSyms = GetSymbols (kmemSymbols, 0);
}


void SasCommand (int cmd)
{
    if (noSyms)
    {
	beep ();
	return;
    }

    if (cmd >= '0' && cmd <= '9')
	sasPort = cmd - '0';
    else if (cmd == ',' || cmd == KEY_UP || cmd == KEY_PREVIOUS)
	sasPort -= (sasPort > 0) ? 1 : 0;
    else if (cmd == '.' || cmd == KEY_DOWN || cmd == KEY_NEXT)
	sasPort += 1;
    else if (cmd == '<' || cmd == KEY_LEFT || cmd == KEY_BEG)
	sasPort = 0;
    else if (cmd == '>' || cmd == KEY_RIGHT || cmd == KEY_END)
	sasPort = LASTPORT;
    else
    {
	sasPort = 0;
	beep ();
    }
}


void DspSasLiterals (WINDOW *win, int tly, int tlx)
{
    static char *header = "---- SAS statistics ";
    int         x2;
    int         y;

    wmove (win, tly, tlx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x2);
    while (x2++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "SAS related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    use_cp (win, cpLIT);
    ClearArea (win, getmaxy (win) - 1, 0, getmaxx (win));
}


void DspSas (WINDOW *win, int tly, int tlx)
{
    struct sas_info     *sip;
    uint                *smcb;
    uint                *svec;
    ulong               *sport;
    uint                count;
    int                 idx;
    int                 num;
    int                 entries;

    if (noSyms)
	return;

    count  = *KTUT (ADDR_sas_physical_units, uint *);
    sip    = KTUT (ADDR_sas_info, struct sas_info *);
    smcb   = KTUT (ADDR_sas_mcb, uint *);
    svec   = KTUT (ADDR_sas_vec, uint *);
    sport  = KTUT (ADDR_sas_port, ulong *);

    entries = getmaxy (win) / 10;
    entries = min (count, entries);

    if (sasPort >= count)
	sasPort = count - 1;

    for (
	num = idx = 0;
	num < entries && idx < count;
	idx++, sip++, smcb++, svec++, sport++
    )
    {
	if (idx < sasPort)
	    continue;

	ShowSasEntry (
	    win, idx, tly + (10 * num), tlx, sip, *sport, *svec, *smcb
	);

	num++;
    }

    wclrtobot (win);
    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

#else
/* Prevent "empty translation unit" warnings from the idiots at X3J11. */
static char ansi_c_idiots = 69;
#endif
