/*
**
**                                S C S I . C
**
**
**      Module:     scsi report
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      21-OCT-1992 AV  Version X1.0, first definition
**      22-OCT-1992 AV  SCSI works fine under SINIX 5.41
**      23-OCT-1992 AV  complete redesign of this module
**      23-OCT-1992 AV  Version 1.0
**      24-OCT-1992 AV  ScsiCommand() was missing, if !defined(MODULE_SCSI)
**      22-OCT-1993 AV  added UnixWare support
**      31-OCT-1993 AV  some more error checking for UnixWare
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**
*/

#include "config.h"

#if defined(MODULE_SCSI)

/*
**  If you want to develop SCSI support on your system, just look to the parts
**  surrounded by #if FEATURE == FEAT_SADM) ... #endif and get the idea.
**  You needn't change anything outside that parts, otherwise report it.
**  Figure out how to do it on your system and send me your feature.
**  And add this port as a feature, like FEAT_SADM.
**
**  Features:
**
**  FEAT_SADM       for SINIX 5.41 SVR4 unix (using sadm interface)
**  FEAT_SDI        for MICROPORT SVR4.0.3 (using sdi interface)
**                  (only some information gathered - needs more work)
**  FEAT_UNIVEL     UnixWare 1.0
**                  As stated in the README file, the scsi code won't work
**                  on my UnixWare. It seems that if you have update3 for
**                  UnixWare installed, then this code should work.
*/
#define FEAT_SADM       1
#define FEAT_SDI        2
#define FEAT_UNIVEL     3

#if !defined(FEATURE)
#   if defined(S_SINIX) || defined(S_SINIX_MIPS)
#       define FEATURE      FEAT_SADM
#   elif defined(S_MICROPORT)
#       define FEATURE      FEAT_SDI
#   elif defined(S_DELL)
#       define FEATURE      FEAT_SDI
#   elif defined(S_UNIVEL)
#       define FEATURE      FEAT_UNIVEL
#   else
#       define FEATURE      FEAT_SDI            /* just try */
#   endif
#endif


#if FEATURE == FEAT_SADM
/*
**  Arrrgh... we need to defines this, <sys/types.h> relies on it
*/
#   define SNI
#endif

#include <curses.h>
#include <panel.h>

#include <sys/types.h>
#include <sys/fcntl.h>

/*
**  files to include for selected feature
*/
#if FEATURE == FEAT_SADM
#   include <sys/scsi.h>
#   include <sys/sadmif.h>
#endif

#if FEATURE == FEAT_SDI
#   include <sys/scsi.h>
#   include <sys/sdi_edt.h>
#endif

#if FEATURE == FEAT_UNIVEL
#   include <sys/scsi.h>
#   include <sys/sdi_edt.h>
#   undef TRUE
#   undef FALSE
#   include <sys/scsicomm.h>
#   include <sys/stat.h>
#endif

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"


#if FEATURE == FEAT_SADM
/*
**  file descriptor for /dev/sadm
*/
static int  sadm_fd = -1;
#endif

#if FEATURE == FEAT_UNIVEL
/*
**  file descriptor for temp node to querry scsi stats
*/
static int  scsi_fd = -1;
#endif


/*
**  switch between flags display and throughput display
*/
static int  displayFlags = 1;
static int  oldFlag      = -99;

static int  noSyms;

typedef struct
{
    unsigned int    ctrlr   : 4;    /* controller # */
    unsigned int    target  : 3;    /* target # */
    unsigned int    lun     : 3;    /* logical unit # */

    unsigned int    pertyp  : 5;    /* Peripheral Device Type */
    unsigned int    devtyp  : 6;    /* Device-Type Modifier */
    unsigned int    ansi    : 3;    /* ANSI-Approved Version */
    unsigned int    perqual : 3;    /* Peripheral Qualifier */
    unsigned int    rmb     : 1;    /* removable medium */
    unsigned int    aenc    : 1;    /* asynchronous event notification capability */

    unsigned int    trmiop  : 1;    /* terminate I/O process */
    unsigned int    reladr  : 1;    /* relative addressing */
    unsigned int    wbus32  : 1;    /* wide bus 32 */
    unsigned int    wbus16  : 1;    /* wide bus 16 */
    unsigned int    sync    : 1;    /* synchronous transfer */
    unsigned int    link    : 1;    /* linked command */
    unsigned int    cmdque  : 1;    /* command queuing */
    unsigned int    sftre   : 1;    /* soft reset */

    char            vendor[9];      /* vendor ident */
    char            product[17];    /* product ident */
    char            revision[5];    /* revision ident */

} save_inq_t;

/*
**  Currently we'll use a fixed array for the SCSI devices information.
**  The size is choosen arbitrarly so adjust it for your needs.
*/
#define MAX_NUM_DEVICES     32

#ifdef NSDIDEV
#   undef   MAX_NUM_DEVICES
#   define  MAX_NUM_DEVICES NSDIDEV
#endif

static save_inq_t   savDevices[MAX_NUM_DEVICES];
static int          numDevices;

#if FEATURE == FEAT_SDI

#define ADDR_sc_edt             kmemSymbols[0].n_value
#define ADDR_sdi_hacnt          kmemSymbols[1].n_value
#define ADDR_aha_edt            kmemSymbols[2].n_value

static struct nlist kmemSymbols[] =
{
    { "sc_edt"              },
    { "sdi_hacnt"           },
    { "aha_edt"             },
    { 0                     },
};
#endif

#ifndef ID_NODEV
#   define ID_NODEV 0x1f
#endif
#ifndef NO_DEV
#   define NO_DEV   ID_NODEV
#endif

/*
**  SCSI-2 Peripheral Device Type to string
*/
static char *DeviceTypeToString (int type)
{
    switch (type)
    {
	case 0x00:      return ("Disk");
	case 0x01:      return ("Tape");
	case 0x02:      return ("Printer");
	case 0x03:      return ("Processor");
	case 0x04:      return ("WORM");
	case 0x05:      return ("ROM");
	case 0x06:      return ("Scanner");
	case 0x07:      return ("OptMemory");
	case 0x08:      return ("MedChanger");
	case 0x09:      return ("ComDevice");
	case 0x0a:      return ("ASC IT8 A");
	case 0x0b:      return ("ASC IT8 B");
	case ID_NODEV:  return ("Unknown");
    }

    return ("Reserved");
}

#define SetCP(bool) \
    if (bool) use_cp (win, cpREVERSE); else use_cp (win, cpINFO)

static void ShowScsiEntry (WINDOW *win, int tly, int tlx, save_inq_t *sav)
{
    use_cp (win, cpINFO);

    mvwprintw (win, tly, tlx +  0, "%3d",     sav->ctrlr);
    mvwprintw (win, tly, tlx +  4, "%3d",     sav->target);
    mvwprintw (win, tly, tlx +  8, "%3d",     sav->lun);
    mvwprintw (win, tly, tlx + 12, "%-10.10s", DeviceTypeToString (sav->pertyp));
    mvwprintw (win, tly, tlx + 23, "%-8.8s",   sav->vendor);
    mvwprintw (win, tly, tlx + 32, "%-16.16s", sav->product);
    mvwprintw (win, tly, tlx + 49, "%-4.4s",   sav->revision);
}

static void ShowScsiFlags (WINDOW *win, int tly, int tlx)
{
    static char *flagHeader = "X PQ ------- FLAGS -------";
    static char *flagFooter = "(X)ANSI (R)MB (A)ENC (T)rmIOP (R)el(A)dr (W)Bus (S)ync (L)ink (C)mdQue (S)ft(R)e";
    save_inq_t  *sav;
    int         i;

    use_cp (win, cpLIT);
    mvwaddstr (win, tly + 1, tlx + 54, flagHeader);
    mvwaddstr (win, getmaxy (win) - 1, tlx, flagFooter);

    use_cp (win, cpINFO);

    for (tly += 2, sav = savDevices, i = 0; i < numDevices; i++, tly++, sav++)
    {
	if (tly >= getmaxy (win) - 1)
	    break;

	mvwprintw (win, tly, tlx + 54, "%1d", sav->ansi);
	mvwprintw (win, tly, tlx + 56, "%2d", sav->perqual);

	SetCP (sav->rmb);
	mvwaddstr (win, tly, tlx + 59, "R");

	SetCP (sav->aenc);
	mvwaddstr (win, tly, tlx + 61, "A");

	SetCP (sav->trmiop);
	mvwaddstr (win, tly, tlx + 63, "T");

	SetCP (sav->reladr);
	mvwaddstr (win, tly, tlx + 65, "RA");

	use_cp (win, cpINFO);
	if (sav->wbus32)
	    mvwaddstr (win, tly, tlx + 68, "W32");
	else if (sav->wbus16)
	    mvwaddstr (win, tly, tlx + 68, "W16");
	else
	    mvwaddstr (win, tly, tlx + 68, "W8 ");

	SetCP (sav->sync);
	mvwaddstr (win, tly, tlx + 72, "S");

	SetCP (sav->link);
	mvwaddstr (win, tly, tlx + 74, "L");

	SetCP (sav->cmdque);
	mvwaddstr (win, tly, tlx + 76, "C");

	SetCP (sav->sftre);
	mvwaddstr (win, tly, tlx + 78, "SR");
    }
}

static void ShowScsiDyn (WINDOW *win, int tly, int tlx)
{
#if FEATURE == FEAT_SADM
    /*
    **  This part just displays some dynamic informations for the
    **  attached devices. Therefore a new (partial!!) header and a totally
    **  new footer will be displayed. Then for every entry in the savDevices
    **  array the information is retrieved and displayed.
    */
    static char *dynHeader = "pr cs qlen oprs KB/s sc as";
    static char *dynFooter = "PRiority Clops/S QLENgth OPRations/Sec SC Active&Stopped";
    save_inq_t          *sav;
    int                 i;
    struct returnname   rn;
    struct returninfo   info;
    double              delta = stat_period_msec / 1000.0;

    use_cp (win, cpLIT);
    mvwaddstr (win, tly + 1, tlx + 54, dynHeader);
    mvwaddstr (win, getmaxy (win) - 1, tlx, dynFooter);
    wclrtoeol (win);

    use_cp (win, cpINFO);

    for (tly += 2, sav = savDevices, i = 0; i < numDevices; i++, tly++, sav++)
    {
	if (tly >= getmaxy (win) - 1)
	    break;

	info.desc.attnr  = -1;
	info.desc.ctrlr  = sav->ctrlr;
	info.desc.target = sav->target;
	info.desc.lun    = sav->lun;

	if (ioctl (sadm_fd, SADM_GETINFO, &info) < 0)
	{
	    wmove (win, tly, tlx + 54);
	    wclrtoeol (win);
	    continue;
	}

	mvwprintw (win, tly, tlx + 54, "%2d", (int) info.prio);
	mvwprintw (win, tly, tlx + 57, "%2d", (int) (((double) info.clops) / delta));
	mvwprintw (win, tly, tlx + 60, "%4d", (int) info.qlen);
	mvwprintw (win, tly, tlx + 65, "%4d", (int) (((double) info.ops) / delta));
	mvwprintw (win, tly, tlx + 70, "%4d", (int) ((((double) info.data) / delta) / 1024.0));
	mvwprintw (win, tly, tlx + 75, "%2d", (int) info.inun);
	mvwprintw (win, tly, tlx + 78, "%c%c", "-CRqr++++"[info.active], info.stopped ? '+' : '-');
    }
#endif
}


/*
**  Inquiry buffer to saved SCSI status. The buf variable points to
**  an array which is at least 36 bytes long (standard INQUIRY buffer).
*/
static void InquiryToSaved (int c, int t, int l, unsigned char *buf, save_inq_t *sav)
{
    (void) memset ((void *) sav, 0, sizeof (save_inq_t));

    sav->ctrlr   = c;
    sav->target  = t;
    sav->lun     = l;

    if (buf != (unsigned char *) 0)
    {
	sav->pertyp  = (*(buf + 0) >> 0) & 0x1f;
	sav->devtyp  = (*(buf + 1) >> 0) & 0x7f;
	sav->ansi    = (*(buf + 2) >> 0) & 0x7;
	sav->perqual = (*(buf + 0) >> 5) & 0x7;
	sav->rmb     = (*(buf + 1) >> 7) & 0x1;
	sav->aenc    = (*(buf + 3) >> 7) & 0x1;
	sav->trmiop  = (*(buf + 3) >> 6) & 0x1;
	sav->reladr  = (*(buf + 7) >> 7) & 0x1;
	sav->wbus32  = (*(buf + 7) >> 6) & 0x1;
	sav->wbus16  = (*(buf + 7) >> 5) & 0x1;
	sav->sync    = (*(buf + 7) >> 4) & 0x1;
	sav->link    = (*(buf + 7) >> 3) & 0x1;
	sav->cmdque  = (*(buf + 7) >> 1) & 0x1;
	sav->sftre   = (*(buf + 7) >> 0) & 0x1;

	(void) strncpy (sav->vendor,   buf + 8,  8);
	(void) strncpy (sav->product,  buf + 16, 16);
	(void) strncpy (sav->revision, buf + 32, 4);
    }
    else
    {
	sav->pertyp = ID_NODEV;
	(void) strncpy (sav->vendor,   "N/A",  8);
	(void) strncpy (sav->product,  "N/A", 16);
	(void) strncpy (sav->revision, "N/A",  4);
    }
}


static int DeviceCompare (const void *pp1, const void *pp2)
{
    save_inq_t  *p1 = (save_inq_t *) pp1;
    save_inq_t  *p2 = (save_inq_t *) pp2;
    int         result;

    /*
    **  sort the SCSI devices:
    **      1) controller number
    **      2) target number
    **      3) logical unit number
    */
    if ((result = p1->ctrlr - p2->ctrlr) == 0)
	if ((result = p1->target - p2->target) == 0)
	    return (p1->lun - p2->lun);

    return (result);
}


/*
**  This functions tries to find all attched SCSI devices and calls
**  InquiryToSaved() on each.
**
**  I actually cannot see any _portable_ way to to this. So you
**  have to check for yourself what your system provides.
**
**  general algorithm:
**
**      forall ctrl in controllers
**          forall targ in targets
**              forall lun in lunets
**                  if (ctrl,targ,lun) is available/valid
**                      if SADM_GETINQUIRY (ctrl,targ,lun) -> buffer
**                          InquiryToSaved (
**                              ctrl, target, lun, (unsigned char *) buffer
**                          )
**
**  alternative:
**
**      SADM_GETATTACHEDUNITS --> att_units
**      forall attnr in att_units
**          SADM_GETINQUIRY (attnr) -> ret
**          InquiryToSaved (
**              ret.ctrl, ret.target, ret.lun, (unsigned char *) ret.buffer
**          )
**
**  The SADM_* ioctl() commands are specific to SINIX 5.41 !!!!
**  Just get the idea.
**
*/

static void ReadScsiInfo ()
{
    numDevices = 0;

#if FEATURE == FEAT_SADM
    /*
    **  Try to determine all attached SCSI devices and fill up the
    **  savDevices array. All display functions uses this array.
    **  An array item itself holds all data derived from an SCSI
    **  INQUIRY command.
    */
    {
	struct returninq    ri;
	struct returnatt    ra;
	int                 i;

	if (ioctl (sadm_fd, SADM_GETATTACHEDUNITS, &ra) < 0)
	    return; /* error occured, just return */

	for (i = 0; i < MAXATTACHABLES; i++)
	{
	    if (ra.att[i] < 0)
		continue;

	    ri.desc.attnr = ra.att[i];
	    ri.desc.ctrlr = -1;
	    if (ioctl (sadm_fd, SADM_GETINQUIRY, &ri) < 0)
	    {
		/*
		**  attached but not online - mark and report it anyway
		*/
		InquiryToSaved (
		    ri.desc.ctrlr, ri.desc.target, ri.desc.lun,
		    (unsigned char *) 0, &savDevices[numDevices]
		);
	    }
	    else
	    {
		/*
		**  now we got an ctrl/target/lun triple, record it
		*/
		InquiryToSaved (
		    ri.desc.ctrlr, ri.desc.target, ri.desc.lun,
		    (unsigned char *) ri.inq, &savDevices[numDevices]
		);
	    }

	    if (++numDevices >= MAX_NUM_DEVICES)
		break;
	}
    }
#endif

#if FEATURE == FEAT_SDI
    /*
    **  Try to determine all attached SCSI devices and fill up the
    **  savDevices array. All display functions uses this array.
    **  An array item itself holds all data derived from an SCSI
    **  INQUIRY command.
    */
    {
	int                 i;
	save_inq_t          *sav;
	long int	    sdi_hacnt;	/* number of host adapters */
	struct scsi_edt     *sc_edt;    /* base ptr for scsi_edt entries */
					/* in array sc_edt[#host adapters][#targets] */

	sc_edt    = KTUT (ADDR_sc_edt, struct scsi_edt *);
	sdi_hacnt = *KTUT (ADDR_sdi_hacnt, long int *);

	for (i = 0; i < sdi_hacnt * MAX_TCS; sc_edt++, i++)
	{
	    if (sc_edt->pdtype == NO_DEV)   /* entry not used */
		continue;

	    sav = &savDevices[numDevices];

	    (void) memset ((void *) sav, 0, sizeof (save_inq_t));

	    sav->ctrlr  = (i / MAX_TCS) & 0x0f;
	    sav->target = (i % MAX_TCS) & 0x07;
	    sav->pertyp = sc_edt->pdtype & 0x1f;
	    (void) strncpy (sav->vendor,  sc_edt->tc_inquiry,      8);
	    (void) strncpy (sav->product, sc_edt->tc_inquiry + 8, 16);

	    if (++numDevices >= MAX_NUM_DEVICES)
		break;
	}
    }
#endif

#if FEATURE == FEAT_UNIVEL
    /*
    **  Try to determine all attached SCSI devices and fill up the
    **  savDevices array. All display functions uses this array.
    **  An array item itself holds all data derived from an SCSI
    **  INQUIRY command.
    */
    {
	int                 i;
	save_inq_t          *sav;
	long int	    sdi_hacnt;	/* number of host adapters */
	struct scsi_edt	    *sc_edt;    /* base ptr for scsi_edt entries */
					/* in array sc_edt[#host adapters][#targets] */
	struct scsi_edt	    *edtbase;
	extern void         *calloc ();

	if (scsi_fd < 0)    /* never opened node */
	    return;

	if (ioctl (scsi_fd, B_HA_CNT, &sdi_hacnt) < 0)
	{
	    perror ("ioctl B_HA_CNT");
    close_and_leave:
	    (void) close (scsi_fd); /* close node so we wont try again */
	    scsi_fd = -1;
	    return;
	}
	edtbase = sc_edt = (struct scsi_edt *) calloc (
	    MAX_TCS * sdi_hacnt, sizeof (struct scsi_edt)
	);
	if (!edtbase)
	{
	    fprintf (stderr, "calloc failure for edtpool\n");
	    goto close_and_leave;
	}

	if (ioctl (scsi_fd, B_REDT, sc_edt) < 0)
	{
	    perror ("ioctl B_REDT");
	    (void) free ((void *) edtbase);
	    goto close_and_leave;
	}

	for (i = 0; i < sdi_hacnt * MAX_TCS; sc_edt++, i++)
	{
	    if (sc_edt->tc_equip == 0)
		continue;

	    sav = &savDevices[numDevices];

	    (void) memset ((void *) sav, 0, sizeof (save_inq_t));

	    sav->pertyp = sc_edt->pdtype & 0x1f;
	    sav->ctrlr  = (i / MAX_TCS) & 0x0f;
	    sav->target = (i % MAX_TCS) & 0x07;
	    (void) strncpy (sav->vendor,  sc_edt->tc_inquiry,      8);
	    (void) strncpy (sav->product, sc_edt->tc_inquiry + 8, 16);

	    if (++numDevices >= MAX_NUM_DEVICES)
		break;
	}
	(void) free ((void *) edtbase);
    }
#endif

    /*
    **  there is no need to care for precedencies, we do it here
    */
    if (numDevices > 0)
	(void) qsort ((char *) savDevices,
	    (unsigned) numDevices, sizeof (save_inq_t), DeviceCompare
	);
}


/*
**   00000000001111111111222222222233333333334444444444555555555566666666667777777777
**   01234567890123456789012345678901234567890123456789012345678901234567890123456789
**00 --------- SCSI configuration ---------------------------------------------------
**01 Ctl Trg LUN DeviceType Vendor   Product          Rev  X PQ ------- FLAGS -------
**02 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**03 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**04 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**05 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**06 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**07 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**08 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**09 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**10 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**11 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**12 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**13 ### ### ### ########## ######## ################ #### # ## R A T RA W## S L C SR
**14 (X)ANSI (R)MB (A)ENC (T)rmIOP (R)el(A)dr (W)Bus (S)ync (L)ink (C)mdQue (S)ft(R)e
*/

void DspScsiLiterals (WINDOW *win, int tly, int tlx)
{
    static char *header = "Ctl Trg LUN DeviceType Vendor   Product          Rev  ";
    int         x;
    int         y;
    int         i;

    use_cp (win, cpBANNER);
    wmove (win, tly, tlx);
    waddstr (win, "---- SCSI configuration ");
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "SCSI related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    /*
    **  re-fetch INQUIRY data for all attached devices
    */
    ReadScsiInfo ();
    oldFlag = -99;

    wclrtobot (win);

    use_cp (win, cpLIT);
    wmove (win, tly + 1, tlx);
    waddstr (win, header);

    /*
    **  display the static (unique) part
    */
    for (tly += 2, i = 0; i < numDevices; i++, tly++)
    {
	if (tly >= getmaxy (win) - 1)
	    break;
	ShowScsiEntry (win, tly, tlx, &savDevices[i]);
    }
}


void DspScsi (WINDOW *win, int tly, int tlx)
{
    if (noSyms)
	return;

    if (oldFlag != displayFlags)
    {
	oldFlag = displayFlags;

	ClearRectangle (win,
	    tly + 2, tlx + 53, getmaxy (win) - 2, getmaxx (win) - 1
	);
    }

    if (displayFlags)
	ShowScsiFlags (win, tly, tlx);
    else
	ShowScsiDyn (win, tly, tlx);

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}


void ScsiCommand (int cmd)
{
    if (cmd == '%')
	displayFlags = !displayFlags;
    else
	beep ();

#if FEATURE == FEAT_SADM
    if (!displayFlags)
	(void) ioctl (sadm_fd, SADM_CLEARSTATS, NULL);
#endif
}


void ScsiInit ()
{
#if FEATURE == FEAT_SADM
    /*
    **  open the sadm clone device. It will provide all neccessary
    **  information.
    */
    if ((sadm_fd = open ("/dev/sadm", O_RDONLY, 0)) < 0)
	noSyms = 1;
    else
	noSyms = 0;
#endif

#if FEATURE == FEAT_SDI
    noSyms = GetSymbols (kmemSymbols, 0);

    if (noSyms)
    {
	if ((ADDR_sc_edt != 0 || ADDR_aha_edt != 0) && ADDR_sdi_hacnt != 0)
	    noSyms = 0;
    }
#endif

#if FEATURE == FEAT_UNIVEL
    int     dev;
    int     fd;
    char    node[] = TEMPNODE;

    noSyms = 1;

    if ((fd = open ("/dev/rdsk/c0t0d0s0", O_RDONLY)) < 0)
    {
	perror ("open /dev/rdsk/c0t0d0s0");
	return;
    }
    if (ioctl (fd, B_GETDEV, &dev) != 0)
    {
	perror ("ioctl");
	(void) close (fd);
	return;
    }

    (void) close (fd);
    (void) mktemp (node);

    if (mknod (node, S_IFCHR | S_IREAD, dev) != 0)
    {
	perror(node);
	return;
    }
    if ((scsi_fd = open (node, O_RDONLY)) < 0)
    {
	perror (node);
	(void) unlink (node);
	return;
    }

    (void) unlink (node);
    noSyms = 0;
#endif
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspScsiLiterals (WINDOW *win, int tly, int tlx)
{
    char    *message = "SCSI info not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void DspScsi (WINDOW *win, int tly, int tlx)
{
}

void ScsiInit ()
{
}

void ScsiCommand (int cmd)
{
}

#endif
