/*
**
**                              S T R E A M . C
**
**
**      Module:     display of STREAMS statistics
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      21-OCT-1992 AV  start port for ARCH_MOT_88k
**      23-OCT-1992 AV  Version 1.0
**      15-NOV-1993 AV  fixed positioning bug in DspStreamLiterals
**
*/

#include "config.h"

#if defined(MODULE_STREAM)

#include <curses.h>
#include <panel.h>

#include <sys/stream.h>
#include <sys/strstat.h>
#include <sys/strsubr.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

#define ADDR_Strinfo                kmemSymbols[0].n_value
#define ADDR_nstrpush               kmemSymbols[1].n_value
#define ADDR_strctlsz               kmemSymbols[2].n_value
#define ADDR_strmsgsz               kmemSymbols[3].n_value
#define ADDR_strst                  kmemSymbols[4].n_value
#define ADDR_strthresh              kmemSymbols[5].n_value
#define ADDR_Strcount               kmemSymbols[6].n_value

static struct nlist kmemSymbols[] =
{
    { "Strinfo"             },
    { "nstrpush"            },
    { "strctlsz"            },
    { "strmsgsz"            },
    { "strst"               },
    { "strthresh"           },
    { "Strcount"            },
    { 0                     },
};


/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  ---- STREAMS statistics --------------------------------------------------------
**  RESOURCE               ALLOC   IN USE    FREE         TOTAL     MAX    FAIL
**  Streams              #######  ####### #######  ############ ####### #######
**  Queues
**  Message blocks
**  Data blocks
**  Link blocks
**  Stream events
**
**  strmsgsz:  #######k
**  strctlsz:  #######k
**  nstrpush:  #######
**  strthresh: #######k
**  Strcount:  #######
*/

#define _STREAM_AL      21
#define _STREAM_IN      30
#define _STREAM_FR      38
#define _STREAM_TO      47
#define _STREAM_MA      60
#define _STREAM_FA      68

#define _STREAM_ADD     11

void DspStreamLiterals (WINDOW *win, int ty, int tx)
{
    static char *header = "RESOURCE               ALLOC   IN USE    FREE         TOTAL     MAX    FAIL";
    int         x;
    int         y;

    wmove (win, ty, tx);
    use_cp (win, cpBANNER);
    waddstr (win, "---- STREAMS statistics ");
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    y = ty + 1;
    wmove (win, y++, tx);
    use_cp (win, cpLIT);
    waddstr (win, header);

    ClearArea (win, getmaxy (win) - 1, 0, getmaxx (win));

    mvwaddstr (win, y++, tx, "Streams       ");
    mvwaddstr (win, y++, tx, "Queues        ");
    mvwaddstr (win, y++, tx, "Message blocks");
    mvwaddstr (win, y++, tx, "Data blocks   ");
    mvwaddstr (win, y++, tx, "Link blocks   ");
    mvwaddstr (win, y++, tx, "Stream events ");
    y++;
    mvwaddstr (win, y++, tx, "strmsgsz: ");
    mvwaddstr (win, y++, tx, "strctlsz: ");
    mvwaddstr (win, y++, tx, "nstrpush: ");
    mvwaddstr (win, y++, tx, "strthresh:");
    mvwaddstr (win, y++, tx, "Strcount:");
} 


void DspStream (WINDOW *win, int ty, int tx)
{
    static struct strstat   strStat;
    struct strstat          *sptr;
    static struct strinfo   strInfo[NDYNAMIC];
    struct strinfo          *siptr;
    int                     x;
    int                     y;

    siptr = KTUT (ADDR_Strinfo, struct strinfo *);
    (void) memcpy ((void *) &strInfo, (void *) siptr, sizeof (strInfo));

    sptr = KTUT (ADDR_strst, struct strstat *);
    (void) memcpy ((void *) &strStat, (void *) sptr, sizeof (strStat));

    use_cp (win, cpINFO);

    y = ty + 2;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_STREAM].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.stream.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_STREAM].sd_cnt - strStat.stream.use);
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.stream.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.stream.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.stream.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.stream.fail);

    y++;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_QUEUE].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.queue.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_QUEUE].sd_cnt - strStat.queue.use);
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.queue.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.queue.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.queue.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.queue.fail);

    y++;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_MSGBLOCK].sd_cnt + strInfo[DYN_MDBBLOCK].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.msgblock.use + strStat.mdbblock.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_MSGBLOCK].sd_cnt + strInfo[DYN_MDBBLOCK].sd_cnt - (strStat.msgblock.use + strStat.mdbblock.use));
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.msgblock.total + strStat.mdbblock.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.msgblock.imax + strStat.mdbblock.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.msgblock.max + strStat.mdbblock.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.msgblock.fail + strStat.mdbblock.fail);

    y++;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_MDBBLOCK].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.mdbblock.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_MDBBLOCK].sd_cnt - strStat.mdbblock.use);
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.mdbblock.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.mdbblock.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.mdbblock.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.mdbblock.fail);

    y++;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_LINKBLK].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.linkblk.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_LINKBLK].sd_cnt - strStat.linkblk.use);
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.linkblk.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.linkblk.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.linkblk.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.linkblk.fail);

    y++;
    mvwprintw (win, y, _STREAM_AL + tx, "%7d",  strInfo[DYN_STREVENT].sd_cnt);
    mvwprintw (win, y, _STREAM_IN + tx, "%7d",  strStat.strevent.use);
    mvwprintw (win, y, _STREAM_FR + tx, "%7d",  strInfo[DYN_STREVENT].sd_cnt - strStat.strevent.use);
    mvwprintw (win, y, _STREAM_TO + tx, "%12d", strStat.strevent.total);
#if defined(S_MOTOROLA_88K)
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.strevent.imax);
#else
    mvwprintw (win, y, _STREAM_MA + tx, "%7d",  strStat.strevent.max);
#endif
    mvwprintw (win, y, _STREAM_FA + tx, "%7d",  strStat.strevent.fail);

    y += 2; x = _STREAM_ADD + tx;
    mvwprintw (win, y++, x, "%7d",  *KTUT (ADDR_strmsgsz, int *));
    mvwprintw (win, y++, x, "%7dk", pagetok (*KTUT (ADDR_strctlsz, int *)));
    mvwprintw (win, y++, x, "%7d",  *KTUT (ADDR_nstrpush, int *));
    mvwprintw (win, y++, x, "%7d",  *KTUT (ADDR_strthresh, int *));
    mvwprintw (win, y++, x, "%7d",  *KTUT (ADDR_Strcount, int *));

    wclrtoeol (win);

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

void StreamInit ()
{
    (void) GetSymbols (kmemSymbols, 1);
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspStreamLiterals (WINDOW *win, int tly, int tlx)
{
    char    *message = "STREAM statistics not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void DspStream (WINDOW *win, int tly, int tlx)
{
}

void StreamInit ()
{
}

#endif
