/*
**
**                             S V R 4 M O N . C
**
**
**      Module:     main module for svr4mon
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      16-OCT-1992 AV  the minfo structure is obsolete in SVR4, use
**                      vminfo instead
**      17-OCT-1992 AV  sysinfo/vminfo layout sligtly changed
**                      added display od absolute values for sysinfo/vminfo
**      17-OCT-1992 AV  added help page (command is now omitted)
**      21-OCT-1992 AV  forget about <sys/ascii.h>, some SVR4 don't support
**                      it :-((
**      21-OCT-1992 AV  added generic SCSI support - see also scsi.c
**      23-OCT-1992 AV  do time calculations now in clock ticks instead
**                      using whole seconds
**      23-OCT-1992 AV  Version 1.0
**      23-OCT-1992 AV  fixed readch,writech display, --> unsigned
**      24-OCT-1992 AV  infoMode is now global
**      25-OCT-1992 AV  added SLIP support
**      28-OCT-1992 AV  use now poll(stdin); command input makes svr4mon to
**                      respond without delay
**      30-NOV-1992 AV  honour the return value of poll()
**      22-OCT-1993 AV  added support for Univel UnixWare 1.0
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**      14-NOV-1993 AV  split update funcs into two funcs: calc and display
**
*/

#include "config.h"
#include "patchlevel.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>

#include <curses.h>
#include <panel.h>

#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <stropts.h>
#include <poll.h>

#include <sys/types.h>
#include <sys/times.h>
#include <sys/lock.h>
#include <sys/utsname.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/sysinfo.h>
#include <sys/swap.h>

#if defined(S_SINIX_MIPS)
#   include <sys/cpupriv.h>
#endif

#include <sys/procset.h>        /* for P_PID */
#include <sys/priocntl.h>
#include <sys/tspriocntl.h>
#include <sys/rtpriocntl.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"
#include "logging.h"

#define ADDR_avenrun            kmemSymbols[0].n_value
#define ADDR_freemem            kmemSymbols[1].n_value
#define ADDR_lbolt              kmemSymbols[2].n_value
#define ADDR_physmem            kmemSymbols[3].n_value
#if defined(S_SINIX_MIPS)
#   define ADDR_cpuprivvir          kmemSymbols[4].n_value
#else
#   define ADDR_sysinfo             kmemSymbols[4].n_value
#   define ADDR_vminfo              kmemSymbols[5].n_value
#endif

static struct nlist kmemSymbols[] =
{
    { "avenrun"             },
#ifdef S_UNIVEL
    { "mem_freemem"         },
#else
    { "freemem"             },
#endif
    { "lbolt"               },
    { "physmem"             },
#if defined(S_SINIX_MIPS)
    { "cpuprivvir"          },
#else
    { "sysinfo"             },
    { "vminfo"              },
#endif
    { 0                     }
};


/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**
**   u386mon xxx.xxx                       PLOCK     INVALID                hh:mm:ss
**  ---- CPU --- tot usr ker brk -- LoadAvg: ---------------------------------------
**   Instant %   ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**   5 Sec Avg % ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**  10 Sec Avg % ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**  ---- Wait -- tot  io pio swp -- (% of real time) -------------------------------
**   Instant %   ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**   5 Sec Avg % ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**  10 Sec Avg % ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
**
*/

static char             *programName;
static char             stdoutbuf[2048];
static int              plock_indicator = 0;
static int              RT_indicator = 0;
static int              banner_free_x;
static long             nowTime;
static long             lastTime;
static long             relTime = 0;
static struct tm        *lcTime;
static struct tms       dummyTMS;
static id_t             rtClass = PC_CLNULL;
static id_t             tsClass = PC_CLNULL;
static int              niced = 0;

#if defined(LOGGING)
static int              kvarLogging    = 0;
static int              kvarSuspended  = 0;
#endif

#define GetTimeMsec()   (times (&dummyTMS) * 10)

enum info_mode_e        infoMode = modeDelta;

#define UROOT           (getuid () == 0)

#define INEXACT_STATE   2
#define INVALID_STATE   5

#define CYCLEmsecDef    2000L
#define CYCLEmsecMax    9000L

static long             CYCLEmsec = CYCLEmsecDef;

static struct sysinfo   v_sysinfo;
static struct sysinfo   v_sysinfo_last;
struct sysinfo          out_sysinfo;
#define sysidelta(x)    \
    (infoMode != modeAbsolute \
	? (out_sysinfo.x = v_sysinfo.x - v_sysinfo_last.x) \
	: (out_sysinfo.x = v_sysinfo.x))

static struct vminfo    v_vminfo;
static struct vminfo    v_vminfo_last;
struct vminfo           out_vminfo;
#define vmidelta(x)     \
    (infoMode != modeAbsolute \
	? (out_vminfo.x = v_vminfo.x - v_vminfo_last.x) \
	: (out_vminfo.x = v_vminfo.x))

static time_t  cpu_last[5];
#define cpudelta(i)    (v_sysinfo.cpu[i] - cpu_last[i])

static time_t  wait_last[3];
#define waitdelta(i)    (v_sysinfo.wait[i] - wait_last[i])

#define SaveCpuAndWait() \
    { \
	(void) memcpy ((void *) cpu_last,  (void *) v_sysinfo.cpu,  sizeof (cpu_last)); \
	(void) memcpy ((void *) wait_last, (void *) v_sysinfo.wait, sizeof (wait_last)); \
    }

#if defined(S_SINIX_MIPS)
#   define ReadSysInfo() \
	{ \
	    cpupriv_t   **a_cpupriv_p;\
	    cpupriv_t   *cpupriv_p; \
	    \
	    a_cpupriv_p = KTUT (ADDR_cpuprivvir, void *); \
	    cpupriv_p   = KTUT (a_cpupriv_p[0], cpupriv_t *); \
	    (void) memcpy ((void *) &v_vminfo, \
		&cpupriv_p->cp_data.cp_d_vminfo, sizeof (struct vminfo)); \
	    (void) memcpy ((void *) &v_sysinfo, \
		&cpupriv_p->cp_data.cp_d_sysinfo, sizeof (struct sysinfo)); \
	}
#   define SaveSysInfo() \
	{ \
	    (void) memcpy ((void *) &sysinfo_last, \
		(void *) &v_sysinfo, sizeof (struct sysinfo)); \
	    (void) memcpy ((void *) &vminfo_last,  \
		(void *) &v_vminfo, sizeof (struct vminfo)); \
	}
#else
#   define ReadSysInfo() \
	{ \
	    (void) memcpy ((void *) &v_vminfo, \
		KTUT (ADDR_vminfo, void *), sizeof (struct vminfo)); \
	    (void) memcpy ((void *) &v_sysinfo, \
		KTUT (ADDR_sysinfo, void *), sizeof (struct sysinfo)); \
	}

#   define SaveSysInfo() \
	{ \
	    (void) memcpy ((void *) &v_sysinfo_last, \
		(void *) &v_sysinfo, sizeof (struct sysinfo)); \
	    (void) memcpy ((void *) &v_vminfo_last, \
		(void *) &v_vminfo,  sizeof (struct vminfo)); \
	}
#endif

static int              stat_period_msec_y = -1;
static int              stat_period_msec_x = -1;

static int              invalidity  = 0;

#define CPU_AVG_MAX     10
static time_t           *cpu_avg[CPU_AVG_MAX];
static time_t           cpu_ticks[5];
time_t                  cpu_percent[5];

#define WAIT_AVG_MAX    10
static time_t           *wait_avg[WAIT_AVG_MAX];
static time_t           wait_ticks[5];
time_t                  wait_percent[3];

long                    memMax;
long                    memFree;
long                    swapMax;
long                    swapFree;


PANEL                   *pscr = 0;
WINDOW                  *wscr = 0;
int                     color_avail = 0;
long                    stat_period_msec;
long                    hertz;

extern WINDOW           *wdet;
extern int              sys_nerr;
extern char             *sys_errlist[];


void Leaving ()
{
    wmove (wscr, getmaxy (wscr) - 1, 0);
    use_cp (wscr, cpLIT);
    wclrtoeol (wscr);

    if (wdet != (WINDOW *) 0)
    {
	wmove (wdet, getmaxy (wdet) - 1, 0);
	use_cp (wdet, cpLIT);
	wclrtoeol (wdet);
    }

    PanelFlush ();
    endwin ();
}

void Leave (int exit_code)
{
    Leaving ();
    exit (exit_code);
}

void LeaveText (char *text, int exit_code)
{
    if (exit_code == 255)
    {
	int     y;
	int     x;

	top_panel (pscr);
	wmove (wscr, MSG_TLY - 3, 0);
	use_cp (wscr, cpHIGH);

	x = 0;
	while (x++ < getmaxx (wscr))
	    waddch (wscr, (chtype) ' ');
	wmove (wscr, MSG_TLY - 2, 0);
	wprintw (wscr, "errno %d", errno);
	if (errno < sys_nerr)
	    wprintw (wscr, ": %s", sys_errlist[errno]);
	getyx (wscr, y, x);
	while (x++ < getmaxx (wscr))
	    waddch (wscr, (chtype) ' ');
    }

    if (text && *text)
	DispMessage (cpHIGH, text);

    Leave (exit_code);
}

static void CaughtSignal (int sig)
{
    Leaving ();

    switch (sig)
    {
	case SIGQUIT:
	case SIGILL:
	case SIGTRAP:
	case SIGIOT:
	case SIGEMT:
	case SIGFPE:
	case SIGBUS:
	case SIGSEGV:
	case SIGSYS:
	    abort();
    }

    exit (200);
}

static void DrwCpuScale (WINDOW *win, int y, int x)
{
    int     x2 = x;

    wmove (win, y, x);
    use_cp (wscr, cpBANNER);

    waddstr (win, "---- CPU --- tot usr ker brk ");
    getyx (win, y, x2);

    while (x2 < getmaxx (win))
	waddch (win, (chtype) '-'), x2++;

    use_cp (wscr, cpLIT);
    wmove (win, y + 1, x);

    if (CYCLEmsec == 1000L)
	waddstr (win," Instant %  ");
    else
	wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec / 1000L));

    wmove (win, y + 2, x);
    wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec * 5 / 1000L));

    wmove (win, y + 3, x);
    wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec * 10 / 1000L));
}

/*
** 000000000011111111112222222222333333333344444444445555555555666666
** 012345678901234567890123456789012345678901234567890123456789012345
** tot usr ker brk
** ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*/

#define _CPUSCALE_TX    0
#define _CPUSCALE_UX    4
#define _CPUSCALE_KX    8
#define _CPUSCALE_BX    12
#define _CPUSCALE_SX    16

static time_t DynCpuScale (WINDOW *win, int y, int x, int width, time_t *per_state)
{
    int     i;
    int     accum = 0;
    time_t  idle = per_state[CPU_IDLE] + per_state[CPU_WAIT];
    time_t  cpu_ticks_total = idle + per_state[CPU_SXBRK] +
		     per_state[CPU_KERNEL] + per_state[CPU_USER];
    time_t  percent_user    = 0;
    time_t  percent_kernel  = 0;
    time_t  percent_break   = 0;
    time_t  percent_busy    = 0;

    if (cpu_ticks_total)
    {
	percent_user   = (per_state[CPU_USER]   * 100) / cpu_ticks_total;
	percent_kernel = (per_state[CPU_KERNEL] * 100) / cpu_ticks_total;
	percent_break  = (per_state[CPU_SXBRK]  * 100) / cpu_ticks_total;
	percent_busy   = percent_user + percent_kernel + percent_break;
    }

    if (!idle)
	percent_busy = 100;

    wmove (win, y, x + _CPUSCALE_TX);
    if (percent_busy < 70)
	use_cp (wscr, cpLOW);
    else if (percent_busy < 90)
	use_cp (wscr, cpMED);
    else
	use_cp (wscr, cpHIGH);
    wprintw (win, "%3ld", percent_busy);

    wmove (win, y, x + _CPUSCALE_UX);
    use_cp (wscr, cpINFO);
    wprintw (win, "%3ld", percent_user);

    wmove (win, y, x + _CPUSCALE_KX);
    wprintw (win, "%3ld", percent_kernel);

    wmove (win, y, x + _CPUSCALE_BX);
    wprintw (win, "%3ld", percent_break);

    wmove (win,y, x + _CPUSCALE_SX);
    use_cp (wscr, cpLOW);
    i = (width * percent_user) / 100;
    accum += i;
    while (i--)
	waddch (win, (chtype) 'u');

    use_cp (wscr, cpMED);
    i = (width * percent_kernel) / 100;
    accum += i;
    while (i--)
	waddch (win, (chtype) 'k');

    use_cp (wscr, cpHIGH);
    i = (width * percent_break) / 100;
    accum += i;
    while (i--)
	waddch (win, (chtype) 'b');

    if (percent_busy > 98 && width - accum > 0)
    {
	waddch (win, (chtype) '*');
	accum++;
    }

    if ((i = width - accum) > 0)
    {
	use_cp (wscr, cpLIT);
	while (i--)
	    waddch (win, (chtype) ' ');
    }

    return (cpu_ticks_total);
}

static void SaveCpuAvg (time_t per_state[], time_t total_ticks)
{
    static int  cpu_avg_init = 0;
    int         i;

    if (!cpu_avg_init)
    {
	for (i = 0; i < CPU_AVG_MAX; i++)
	    (void) memcpy (cpu_avg[i], per_state, sizeof (time_t) * 5);
	cpu_avg_init = 1;
    }
    else
    {
	for (i = 0; i < CPU_AVG_MAX - 1; i++)
	    (void) memcpy (cpu_avg[i], cpu_avg[i + 1], sizeof (time_t) * 5);
	(void) memcpy (cpu_avg[i], per_state, sizeof (time_t) * 5);
    }

#if defined(LOGGING)
    /*
    **  If we are logging, then we need to save the cpu *averages*
    **  so that they can be output.
    */
    cpu_percent[CPU_USER]   = (per_state[CPU_USER]   * 100) / total_ticks;
    cpu_percent[CPU_KERNEL] = (per_state[CPU_KERNEL] * 100) / total_ticks;
    cpu_percent[CPU_SXBRK]  = (per_state[CPU_SXBRK]  * 100) / total_ticks;
    cpu_percent[CPU_IDLE]   = (per_state[CPU_IDLE]   * 100) / total_ticks;
    cpu_percent[CPU_WAIT]   = (per_state[CPU_WAIT]   * 100) / total_ticks;
#endif
}

static void GetCpuAvg (time_t cpu_ticks[], int period)
{
    int     iperiod = CPU_AVG_MAX;
    int     count = period;
    int     istate;

    for (istate = 0; istate < 5; istate++)
	cpu_ticks[istate] = 0;

    while (count--)
    {
	iperiod--;
	for (istate = 0; istate < 5; istate++)
	    cpu_ticks[istate] += cpu_avg[iperiod][istate];
    }

    for (istate = 0; istate < 5; istate++)
	cpu_ticks[istate] /= period;
}

static void DrwWaitScale (WINDOW *win, int y, int x)
{
    int     x2 = x;

    wmove (win, y, x);
    use_cp (wscr, cpBANNER);

    waddstr (win, "---- Wait -- tot  io pio swp -- (% of real time) ");
    getyx (win, y, x2);

    while (x2 < getmaxx (win))
	waddch (win, (chtype) '-'), x2++;

    use_cp (wscr, cpLIT);
    wmove (win, y + 1, x);

    if (CYCLEmsec == 1000L)
	waddstr (win, " Instant %  ");
    else
	wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec / 1000L));

    wmove (win, y + 2, x);
    wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec * 5 / 1000L));

    wmove (win, y + 3, x);
    wprintw (win, "%2d Sec Avg %%", (int)(CYCLEmsec * 10 / 1000L));
}

static void DrwSysInfo (WINDOW *win, int y, int x)
{
    wmove (win, y, x);
    use_cp (wscr, cpBANNER);
    waddstr (win, "---- Sysinfo/VMinfo ---");
    getyx (win, stat_period_msec_y, stat_period_msec_x);
}

/*
**  000000000011111111112222222222333333333344444444445555555555666666
**  012345678901234567890123456789012345678901234567890123456789012345
**  tot  io pio swp
**  ### ### ### ### xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*/
#define _WAITSCALE_TX   0
#define _WAITSCALE_IX   4
#define _WAITSCALE_PX   8
#define _WAITSCALE_WX   12
#define _WAITSCALE_SX   16

static time_t UpdWaitScale (WINDOW *win, int y, int x, int width, time_t *per_state, time_t total_ticks)
{
    int     i;
    int     accum = 0;
    time_t  percent_io = 0L;
    time_t  percent_swap = 0L;
    time_t  percent_pio = 0L;
    time_t  percent_total_wait;
    time_t  total_wait;

    total_wait = per_state[W_IO] + per_state[W_SWAP] + per_state[W_PIO];
    if (total_ticks < total_wait)
	total_ticks = total_wait;

    if (total_ticks)
    {
	percent_io   = (per_state[W_IO]   * 100) / total_ticks;
	percent_pio  = (per_state[W_PIO]  * 100) / total_ticks;
	percent_swap = (per_state[W_SWAP] * 100) / total_ticks;
    }

    wmove (win, y, x + _WAITSCALE_TX);
    percent_total_wait = percent_io + percent_swap + percent_pio;

    if (percent_total_wait < 30)
	use_cp (wscr, cpLOW);
    else if (percent_total_wait < 50)
	use_cp (wscr, cpMED);
    else
	use_cp (wscr, cpHIGH);
    wprintw (win, "%3ld", percent_total_wait);

    use_cp (wscr, cpINFO);
    wmove (win, y, x + _WAITSCALE_IX);
    wprintw (win, "%3ld", percent_io);

    wmove (win, y, x + _WAITSCALE_PX);
    wprintw (win, "%3ld", percent_pio);

    wmove (win, y, x + _WAITSCALE_WX);
    wprintw (win, "%3ld", percent_swap);

    wmove (win, y, x + _WAITSCALE_SX);
    use_cp (wscr, cpLOW);
    i = (width * percent_io) / 100;
    accum += i;

    while (i--)
	waddch (win, (chtype) 'i');

    use_cp (wscr, cpMED);
    i = (width * percent_pio) / 100;
    accum += i;
    while (i--)
	waddch (win, (chtype) 'p');

    use_cp (wscr, cpHIGH);
    i = (width * percent_swap) / 100;
    accum += i;

    while (i--)
	waddch (win, (chtype) 's');

    if ((i = width - accum) > 0)
    {
	use_cp (wscr, cpLIT);
	while (i--)
	    waddch (win, (chtype) ' ');
    }
}

static void SaveWaitAvg (time_t per_state[], time_t total_ticks)
{
    static int  wait_avg_init = 0;
    int         i;

    if (!wait_avg_init)
    {
	for (i = 0; i < WAIT_AVG_MAX; i++)
	    (void) memcpy (wait_avg[i], per_state, sizeof (time_t) * 3);
	wait_avg_init = 1;
    }
    else
    {
	for (i = 0; i < WAIT_AVG_MAX - 1; i++)
	    (void) memcpy (wait_avg[i], wait_avg[i + 1], sizeof (time_t) * 3);
	(void) memcpy (wait_avg[i], per_state, sizeof (time_t) * 3);
    }

#if defined(LOGGING)
    /*
    **  Save wait percentages.
    */
    wait_percent[W_IO]   = per_state[W_IO]   / total_ticks;
    wait_percent[W_SWAP] = per_state[W_SWAP] / total_ticks;
    wait_percent[W_PIO]  = per_state[W_PIO]  / total_ticks;
#endif
}


static void GetWaitAvg(time_t wait_ticks[], int period)
{
    int     iperiod = WAIT_AVG_MAX;
    int     istate;
    int     count = period;

    for (istate = 0; istate < 3; istate++)
	wait_ticks[istate] = 0;

    while (count--)
    {
	iperiod--;
	for (istate = 0; istate < 3; istate++)
	    wait_ticks[istate] += wait_avg[iperiod][istate];
    }

    for (istate = 0; istate < 3; istate++)
	wait_ticks[istate] /= period;
}

static void Usage ()
{
    static char *usage_str[]=
    {
	"-l    lock process into memory (if root)",
	"-L    run process in RT scheduling class (if root)",
	"-p    begin with short ps display",
	"-P    begin with long ps display (if >38 line screen)",
	"-e    begin with extra display",
	"-s    begin with streams display",
	"-a    begin with system architecture and devices display",
	"-h    begin with hack display",
	"-c    begin with SCSI display",
	"-t    begin with tcp display",
	"-i    begin with ip display",
	"-z    begin with SLIP display",
	"-x    begin with serial ports display",
	"-?    begin with help display",
	"-A    show absolute values",
	"-D    show delta values",
	"-R    show relative values",
#if defined(LOGGING)
	"-C    config file for logging",
	"-o    output file to write logging information",
#endif
	(char *) 0
    };
    char    **cpptr = usage_str;

    fprintf (stderr, "svr4mon version %s, Copyright (C) 1992,1993 Andreas Vogel\n", VERSION);
    fprintf (stderr, "This is free software; see the source for copying conditions.\n");
    fprintf (stderr, "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n");
    fprintf (stderr, "PARTICULAR PURPOSE.\n");
    fputs ("\n", stderr);

    fprintf (stderr, "usage: %s [-l] [-L] [-A|-D|-R]\n", programName);
    fprintf (stderr, "       %.*s [-p|-P|-c|-e|-s|-h|-a|-t|-i|-x|-z|-?]\n", strlen (programName), "          ");
#if defined(LOGGING)
    fprintf (stderr, "       %.*s [-C file -o file]\n", strlen (programName), "           ");
#endif
    fputs ("\n", stderr);

    while (*cpptr)
	(void) fprintf (stderr, "    %s\n", *cpptr++);
    exit (1);
}


static void DrwMainFrame ()
{
    struct utsname  utsname;
    int             x;
    int             y;

    wmove (wscr, 0, 0);
    use_cp (wscr, cpBANNER);
    wprintw (wscr, "%s-%s", programName, VERSION);

    if (uname (&utsname) == -1)
	LeaveText ("uname failed", 255);

    wprintw (wscr," S:%s R:%s V:%s N:%s ",
	utsname.sysname, utsname.release, utsname.version, utsname.nodename
    );

    getyx (wscr, y, x);
    banner_free_x = x + 2;

    while (x < getmaxx (wscr))
	waddch (wscr, (chtype) ' '), x++;

    if (plock_indicator)
    {
	wmove (wscr, 0, banner_free_x);
	use_cp (wscr, cpMED);
	waddstr (wscr, " PLOCK ");
	use_cp (wscr, cpBANNER);
    }

    if (RT_indicator)
    {
	wmove (wscr, 0, banner_free_x + 16);
	use_cp (wscr, cpMED);
	waddstr (wscr, " RT ");
	use_cp (wscr, cpBANNER);
    }

#if defined(LOGGING)
    if (kvarLogging)
    {
	if (kvarSuspended)
	{
	    wmove (wscr, 0, banner_free_x + 20);
	    use_cp (wscr, cpMED);
	    waddstr (wscr, " LS ");
	    use_cp (wscr, cpBANNER);
	}
	else
	{
	    wmove (wscr, 0, banner_free_x + 20);
	    use_cp (wscr, cpMED);
	    waddstr (wscr, " LR ");
	    use_cp (wscr, cpBANNER);
	}
    }
#endif

    wmove (wscr, 3, 0);
    use_cp (wscr, cpMED);
    waddstr (wscr, "WAIT");
    PanelFlush();
}

static char *GetAgeOfSystem ()
{
    static char     timeStr[32];
    int             lbolt;
    int             days, hrs, mins;

    lbolt = *KTUT (ADDR_lbolt, int *);

    lbolt /= (60 * hertz);
    days  = lbolt / (long) (60 * 24); lbolt %= (long) (60 * 24);
    hrs   = lbolt / (long) 60; lbolt %= (long) 60;
    mins  = lbolt;

    (void) sprintf (timeStr, "%4dd %2dh %2dm", days, hrs, mins);

    return (timeStr);
}

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  ---- Sysinfo/VMinfo --- (last ##### msec activity) - ABS - Timer ####h ##m ##s -
**  bread  ######### readch########### pswitch######## vfault ######## swpout ######
**  bwrite ######### writch########### syscall######## pgrec  ######## pswpout######
**  lread  ######### rawch ########### sysread######## pfault ######## swpin  ######
**  lwrite ######### canch ########### syswrit######## xsfrec ######## pswpin ######
**  phread ######### outch ########### sysfork######## xifrec ######## dfree  ######
**  phwrite######### msg   ########### sysexec######## pgin   ######## scan   ######
**  swapin ######### sema  ###########                 pgpgin ########
**  swapout######### maxmem   #######k runque ######## pgout  ########
**  bswapin######### freemem  #######k runocc ######## pgpgout########
**  bswpout######### mem used #######% swpque ######## sftlock########
**  iget   ######### maxswap  #######k swpocc ######## pnpflt ########
**  namei  ######### freeswap #######k                 wrtflt ######## Age of system
**  dirblk ######### swap used#######%                                 ####d ##h ##m
*/

void DispTimer (WINDOW *win, int y, int x, int max)
{
    wmove (win, y, x);

    if (infoMode == modeDelta)
    {
	use_cp (win, cpBANNER);
	waddstr (win, " (last ");
	if (stat_period_msec > CYCLEmsec + 1000L)
	    use_cp (win, cpREVERSE);
	else if (stat_period_msec > CYCLEmsec + 500L)
	    use_cp (win, cpBANWARN);
	wprintw (win, "%ld ", stat_period_msec);
	use_cp (win, cpBANNER);
	waddstr (win, "msec activity) ");
    }
    else if (infoMode == modeAbsolute)
    {
	use_cp (win, cpBANNER);
	waddstr (win, " Absolute (since boot time) ");
    }
    else
    {
	time_t  hh, mm, ss;
	time_t  delta = (nowTime - relTime) / 1000;

	use_cp (win, cpBANNER);
	if (relTime <= 0 || delta < 0)
	    waddstr (win, " Relative (ERROR) ");
	else
	{
	    hh = delta / (60L * 60L); delta -= hh * (60L * 60L);
	    mm = delta / 60L; delta -= mm * 60L;
	    ss = delta;

	    wprintw (win, " Relative (elapsed  %ldh %2ldm %2lds) ", hh, mm, ss);
	}
    }

    use_cp (win, cpBANNER);
    getyx (win, y, x);
    if (max == -1)
	max = getmaxx (win);
    while (x++ < max)
	waddch (win, (chtype) '-');
}


static void UpdSysInfo ()
{
#define MSFILES 16

    static struct SwapTab
    {
	int             swt_n;
	struct swapent  swt_ent[MSFILES];
    } swapTable;
    static int  swapInited = 0;
    int         ret;
    int         i;
    long        memMax;
    long        memFree;
    long        swapMax;
    long        swapFree;
    int         y;
    int         x;

    DispTimer (wscr, stat_period_msec_y, stat_period_msec_x, -1);

    memMax  = pagetok (*KTUT (ADDR_physmem, int *));
    memFree = pagetok (*KTUT (ADDR_freemem, int *));

    if (swapInited == 0)
    {
	swapTable.swt_n = MSFILES;

	for (i = 0; i < swapTable.swt_n; i++)
	{
	    swapTable.swt_ent[i].ste_path = (char *) malloc (MAXPATHLEN);
	    if (swapTable.swt_ent[i].ste_path == NULL)
		LeaveText ("SwapSetup: Error malloc", 255);
	}
	swapInited = 1;
    }

    if ((ret = swapctl (SC_LIST, &swapTable)) <= 0)
    {
	swapMax = swapFree = 99999;   /* Unknown */
    }
    else
    {
	swapMax = swapFree = 0;

	for (i = 0; i < ret; i++)
	{
	    swapMax  += swapTable.swt_ent[i].ste_pages;
	    swapFree += swapTable.swt_ent[i].ste_free;
	}

	swapMax  = pagetok (swapMax);
	swapFree = pagetok (swapFree);
    }

    use_cp (wscr, cpINFO);
    y = PER_SEC_TLY + 1;

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "bread  ", "%9ld", sysidelta (bread));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "bwrite ", "%9ld", sysidelta (bwrite));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "lread  ", "%9ld", sysidelta (lread));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "lwrite ", "%9ld", sysidelta (lwrite));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "phread ", "%9ld", sysidelta (phread));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "phwrite", "%9ld", sysidelta (phwrite));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "swapin ", "%9ld", sysidelta (swapin));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "swapout", "%9ld", sysidelta (swapout));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "bswapin", "%9ld", sysidelta (bswapin));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "bswpout", "%9ld", sysidelta (bswapout));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "iget   ", "%9ld", sysidelta (iget));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "namei  ", "%9ld", sysidelta (namei));

    wmove (wscr, y++, PER_SEC1_TLX);
    DispInfoLong (wscr,   "dirblk ", "%9ld", sysidelta (dirblk));

    y = PER_SEC_TLY + 1;
    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "readch", "%11lu", sysidelta (readch));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "writch", "%11lu", sysidelta (writech));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "rawch ", "%11lu", sysidelta (rawch));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "canch ", "%11lu", sysidelta (canch));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "outch ", "%11lu", sysidelta (outch));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "msg   ", "%11lu", sysidelta (msg));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "sema  ", "%11lu", sysidelta (sema));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr, "maxmem   ",   "%7ldk", memMax);

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "freemem  ", "%7ldk", memFree);

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoInt (wscr,    "mem used ", "%7d%%", 100 - (int)((memFree * 100) / memMax));

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr, "maxswap  ",   "%7ldk", swapMax);

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoLong (wscr,   "freeswap ", "%7ldk", swapFree);

    wmove (wscr, y++, PER_SEC2_TLX);
    DispInfoInt (wscr,    "swap used", "%7d%%",  100 - (int)((swapFree * 100) / swapMax));

    y = PER_SEC_TLY + 1;
    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "pswitch", "%8ld", sysidelta (pswitch));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "syscall", "%8ld", sysidelta (syscall));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "sysread", "%8ld", sysidelta (sysread));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "syswrit", "%8ld", sysidelta (syswrite));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "sysfork", "%8ld", sysidelta (sysfork));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "sysexec", "%8ld", sysidelta (sysexec));

    y++;
    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "runque ", "%8ld", sysidelta (runque));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "runocc ", "%8ld", sysidelta (runocc));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "swpque ", "%8ld", sysidelta (swpque));

    wmove (wscr, y++, PER_SEC3_TLX);
    DispInfoLong (wscr,   "swpocc ", "%8ld", sysidelta (swpocc));

    y = PER_SEC_TLY + 1;
    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "vfault ", "%8ld", vmidelta (v_vfault));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pgrec  ", "%8ld", vmidelta (v_pgrec));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pfault ", "%8ld", vmidelta (v_pfault));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "xsfrec ", "%8ld", vmidelta (v_xsfrec));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "xifrec ", "%8ld", vmidelta (v_xifrec));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pgin   ", "%8ld", vmidelta (v_pgin));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pgpgin ", "%8ld", vmidelta (v_pgpgin));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pgout  ", "%8ld", vmidelta (v_pgout));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pgpgout", "%8ld", vmidelta (v_pgpgout));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "sftlock", "%8ld", vmidelta (v_sftlock));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "pnpflt ", "%8ld", sysidelta (pnpfault));

    wmove (wscr, y++, PER_SEC4_TLX);
    DispInfoLong (wscr,   "wrtflt ", "%8ld", sysidelta (wrtfault));

    y = PER_SEC_TLY + 1;
    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "swpout ", "%6ld", vmidelta (v_swpout));

    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "pswpout", "%6ld", vmidelta (v_pswpout));

    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "swpin  ", "%6ld", vmidelta (v_swpin));

    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "pswpin ", "%6ld", vmidelta (v_pswpin));

    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "dfree  ", "%6ld", vmidelta (v_dfree));

    wmove (wscr, y++, PER_SEC5_TLX);
    DispInfoLong (wscr,   "scan   ", "%6ld", vmidelta (v_scan));

    y = PER_SEC_TLY + 12;

    use_cp (wscr, cpBANNER);
    mvwaddstr (wscr, y++, PER_SEC5_TLX, "Age of system");

    use_cp (wscr, cpINFO);
    mvwaddstr (wscr, y++, PER_SEC5_TLX, GetAgeOfSystem ());

    use_cp (wscr, cpBANNER);
    mvwaddstr (wscr, 1, 31, " LoadAvg: ");
    wprintw (wscr, "%.2f %.2f %.2f ",
	(double) ((*KTUT(ADDR_avenrun + 0, long *)) / (double) (1 << 8)),
	(double) ((*KTUT(ADDR_avenrun + 4, long *)) / (double) (1 << 8)),
	(double) ((*KTUT(ADDR_avenrun + 8, long *)) / (double) (1 << 8))
    );
    getyx (wscr, y, x);
    while (x++ < getmaxx (wscr))
	waddch (wscr, (chtype) '-');

    wmove (wscr, getmaxy (wscr) - 1, getmaxx (wscr) - 1);
}


static int ReadUserCommand ()
{
    chtype  cmd = 0;

    switch (cmd = wgetch (wscr))
    {
	case 'L' & 0x1F:                /* ^L */
	case 'R' & 0x1F:                /* ^R */
	case KEY_REFRESH:
	    clearok (wscr, TRUE);
	    touchwin (wscr);
	    wrefresh (wscr);
	    if (wdet)
	    {
		touchwin (wdet);
		wrefresh (wdet);
	    }
	    break;

	case 'R':
	    infoMode = modeRelative;
	    SaveSysInfo ();
	    DetailModeChange ();
	    relTime = nowTime;
	    break;

	case 'A':
	    if (infoMode != modeAbsolute)
	    {
		infoMode = modeAbsolute;
		DetailModeChange ();
	    }
	    else
		beep ();
	    break;

	case 'D':
	    if (infoMode != modeDelta)
	    {
		infoMode = modeDelta;
		SaveSysInfo ();
		DetailModeChange ();
	    }
	    else
		beep ();
	    break;

	case 'l':
	    if(!plock_indicator && UROOT && !plock (PROCLOCK))
	    {
		plock_indicator = 1;
		wmove (wscr, 0, banner_free_x);
		use_cp (wscr, cpMED);
		waddstr (wscr, " PLOCK ");
		nice (-5);
		niced = 1;
	    }
	    else
		beep ();
	    break;

	case 'L':
	    if(!RT_indicator && UROOT && rtClass != PC_CLNULL)
	    {
		pcparms_t   pcparms;
		rtparms_t   *rtptr = (rtparms_t *) pcparms.pc_clparms;

		pcparms.pc_cid = rtClass;
		rtptr->rt_pri = 0;
		rtptr->rt_tqnsecs = RT_TQDEF;

		if (priocntl (P_PID, P_MYID, PC_SETPARMS, &pcparms) == 0)
		{
		    RT_indicator = 1;
		    wmove (wscr, 0, banner_free_x + 16);
		    use_cp (wscr, cpMED);
		    waddstr (wscr, " RT ");
		    niced = 0;
		}
		else
		    beep ();
	    }
	    else
		beep ();
	    break;

	case 'u':
	    if (plock_indicator && !plock (UNLOCK))
	    {
		plock_indicator = 0;
		wmove (wscr, 0, banner_free_x);
		use_cp (wscr, cpBANNER);
		waddstr (wscr, "       ");
		if (niced && !RT_indicator)
		    nice (5);
		niced = 0;
	    }
	    else
		beep ();
	    break;

	case 'U':
	    if (RT_indicator && tsClass != PC_CLNULL)
	    {
		pcparms_t   pcparms;
		tsparms_t   *tsptr = (tsparms_t *) pcparms.pc_clparms;

		pcparms.pc_cid = tsClass;
		tsptr->ts_uprilim = TS_NOCHANGE;
		tsptr->ts_upri = TS_NOCHANGE;

		if (priocntl (P_PID, P_MYID, PC_SETPARMS, &pcparms) == 0)
		{
		    RT_indicator = 0;
		    wmove (wscr, 0, banner_free_x + 16);
		    use_cp (wscr, cpBANNER);
		    waddstr (wscr, "    ");
		    niced = 0;
		}
		else
		    beep ();
	    }
	    else
		beep ();
	    break;

	case '+':
	    if (CYCLEmsec < CYCLEmsecMax)
	    {
		invalidity += (INEXACT_STATE * (CYCLEmsec + 1000L)) /
							CYCLEmsec;
		CYCLEmsec += 1000L;
		DrwCpuScale (wscr, CPUSCALE_TLY, 0);
		DrwWaitScale (wscr, WAITSCALE_TLY, 0);
	    }
	    else
		beep ();
	    break;

	case '-':
	    if (CYCLEmsec > 1000L)
	    {
		CYCLEmsec -= 1000L;
		DrwCpuScale (wscr, CPUSCALE_TLY, 0);
		DrwWaitScale (wscr, WAITSCALE_TLY, 0);
	    }
	    else
		beep();
	    break;

	case '|':
#if defined(LOGGING)
	    if (kvarLogging)
	    {
		if (kvarSuspended)
		{
		    kvar_resume ();
		    kvarSuspended = 0;
		    wmove (wscr, 0, banner_free_x + 20);
		    use_cp (wscr, cpMED);
		    waddstr (wscr, " LR ");
		    use_cp (wscr, cpBANNER);
		}
		else
		{
		    kvar_suspend ();
		    kvarSuspended = 1;
		    wmove (wscr, 0, banner_free_x + 20);
		    use_cp (wscr, cpMED);
		    waddstr (wscr, " LS ");
		    use_cp (wscr, cpBANNER);
		}
	    }
	    else
#endif
		beep();
	    break;

	case 'q':
	case KEY_EXIT:
	case KEY_END:
	    return (0);

	case 0:
	case -1:
	    break;

	default:
	    DetailCommand (cmd);
	    break;
    }

    return (1);
}

static void CheckForValidity ()
{
    int     i;

    /* heuristic validity determination */
    wmove (wscr, 0, banner_free_x + 7);

    if ((i = (stat_period_msec > CYCLEmsec + 3000L)) || invalidity > INVALID_STATE)
    {
	use_cp (wscr, cpHIGH);
	waddstr (wscr, " INVALID ");
	if (i)
	{
	    i = (stat_period_msec - CYCLEmsec) / 1000;
	    if (i > 4)
		i = 4;
	    invalidity += INVALID_STATE + i;
	}
    }
    else if ((i = (stat_period_msec > CYCLEmsec + 1500L)) || invalidity > INEXACT_STATE)
    {
	use_cp (wscr, cpMED);
	waddstr (wscr, " INEXACT ");
	if (i)
	    invalidity += INEXACT_STATE;
    }

    if (invalidity && !(--invalidity))
    {
	use_cp (wscr, cpBANNER);
	waddstr (wscr, "         ");
    }
}


/*ARGSUSED*/
main (int argc, char **argv, char **envp)
{
    chtype          initial_cmd = 0;
    int             errflg      = 0;
    int             i;
    time_t          total_ticks;
    time_t          nowInSecs;
    clock_t         startClock;
    clock_t         endClock;
    struct pollfd   pollFd[1];
    pcinfo_t        pcinfo;

#if defined(LOGGING)
    char            *log_filename  = (char *) 0;
    char            *conf_filename = (char *) 0;
    kvar_symbol_t   *kvar_symbols  = (kvar_symbol_t *) 0;
#endif

    programName = basename (argv[0]);

    pollFd[0].fd     = 0;
    pollFd[0].events = POLLIN;

    pcinfo.pc_cid = PC_CLNULL;
    (void) strncpy (pcinfo.pc_clname, "TS", PC_CLNMSZ);
    if (priocntl (0, 0, PC_GETCID, &pcinfo) == -1)
    {
	perror ("priocntl for TS");
	exit (1);
    }
    tsClass = pcinfo.pc_cid;

    pcinfo.pc_cid = PC_CLNULL;
    (void) strncpy (pcinfo.pc_clname, "RT", PC_CLNMSZ);
    if (priocntl (0, 0, PC_GETCID, &pcinfo) == -1)
    {
	perror ("priocntl for RT");
	exit (1);
    }
    rtClass = pcinfo.pc_cid;

    /*
    **  curses works better if standard output is fully buffered
    */
    (void) setvbuf (stdout, stdoutbuf, _IOFBF, sizeof (stdoutbuf));

    hertz = sysconf (_SC_CLK_TCK);

    /*
    **  check out command line
    */
    opterr = 0;
    while ((i = getopt (argc, argv, "DALlPcpshaextizo:C:")) != -1)
    {
	switch (i)
	{
	    case 'P':
	    case 'p':
	    case 'c':
	    case 's':
	    case 'h':
	    case 'a':
	    case 'x':
	    case 't':
	    case 'e':
	    case 'i':
	    case 'z':
		initial_cmd = (chtype) i;
		break;
	    case 'l':
		plock_indicator = 1;
		break;
	    case 'L':
		RT_indicator = 1;
		break;
	    case 'A':
		infoMode = modeAbsolute;
		break;
	    case 'D':
		infoMode = modeDelta;
		break;
#if defined(LOGGING)
	    case 'o':
		if ((log_filename = strdup (optarg)) == (char *) NULL)
		{
		    (void) fprintf (stderr, "Cannot duplicate log filename\n");
		    exit (1);
		}
		kvarLogging++;
		break;
	    case 'C':
		if ((conf_filename = strdup (optarg)) == (char *) NULL)
		{
		    (void) fprintf (stderr, "Cannot duplicate configuration filename.\n");
		    exit (1);
		}
		kvarLogging++;
		break;
#endif
	    case '?':
	    default:
		if ((optopt & 0x7f) != '?')
		    errflg++;
		else
		    initial_cmd = (chtype) i;
		break;
	}
    }

#if defined(LOGGING)
    if (kvarLogging == 1)
    {
	(void) fprintf (stderr, "You have to specify -o and -C together.\n");
	errflg++;
    }
#endif

    if (errflg || optind != argc)
	Usage ();

    /*
    **  if man wants to plock() try it; fail silently if non-root
    */
    if (plock_indicator)
    {
	if (UROOT && !plock (PROCLOCK))
	{
	    if (niced)
		nice (-5);
	}
	else
	    plock_indicator = 0;
    }

    if (RT_indicator)
    {
	if (UROOT && rtClass != PC_CLNULL)
	{
	    pcparms_t   pcparms;
	    rtparms_t   *rtptr = (rtparms_t *) pcparms.pc_clparms;

	    pcparms.pc_cid = rtClass;
	    rtptr->rt_pri = 0;
	    rtptr->rt_tqnsecs = RT_TQDEF;

	    if (priocntl (P_PID, P_MYID, PC_SETPARMS, &pcparms) == -1)
		RT_indicator = 0;
	    else
		niced = 0;
	}
	else
	    RT_indicator = 0;
    }

#if defined(LOGGING)
    /*
    **  Initialize logging if necessary.
    */
    if (kvarLogging)
    {
	if ((kvar_symbols = kvars_config_init (log_filename, conf_filename)) == (kvar_symbol_t *) NULL)
	{
	    (void) fprintf (stderr, "Logging initialization failed.\n");
	    exit (1);
	}
    }
#endif

    /*
    ** initialize curses environment
    */
    if(!initscr ())
    {
	(void) fprintf (stderr, "curses init failed\n");
	exit (1);
    }

#if defined(COLOR_PAIR)
    color_avail = has_colors ();
#endif

    clear ();
    refresh ();

    if (LINES < 24 || COLS < 80)
    {
	waddstr (stdscr, "\n\n\nNeed at least 80x24 screen\n\n");
	refresh ();
	endwin ();
	exit (1);
    }

    noecho ();
    keypad (stdscr, TRUE);
    typeahead (-1);
    cbreak ();

#if defined(COLOR_PAIR)
    if (color_avail)
    {
	start_color ();
	init_pair (cpLIT,     cBLU, cBLK);
	init_pair (cpINFO,    cGRN, cBLK);
	init_pair (cpLOW,     cLTG, cBLK);
	init_pair (cpMED,     cYEL, cBLK);
	init_pair (cpHIGH,    cRED, cBLK);
	init_pair (cpBANNER,  cBLK, cWHT);
	init_pair (cpREVERSE, cRED, cWHT);
	init_pair (cpBANWARN, cBLU, cWHT);
    }
#endif

    if (!(pscr = MakePanel (LINES, COLS, 0, 0, "main")))
    {
	addstr ("cannot make screen panel");
	refresh ();
	endwin ();
	exit (1);
    }
    wscr = panel_window (pscr);
    top_panel (pscr);

    nodelay (wscr, TRUE);
    keypad (wscr, TRUE);

    /*
    ** catch signals that can Leave our tty in disarray
    */
    for (i = SIGHUP; i < SIGSYS; i++)
	signal (i, CaughtSignal);

    /*
    ** allocate memory for cpu time array averaging buckets
    */
    for (i = 0; i < CPU_AVG_MAX; i++)
    {
	if (!(cpu_avg[i] = (time_t *) malloc (sizeof (time_t) * 5)))
	    LeaveText ("cannot alloc memory for cpu avg arrays", 1);
    }

    /*
    ** allocate memory for wait time array averaging buckets
    */
    for (i = 0; i < WAIT_AVG_MAX; i++)
    {
	if (!(wait_avg[i] = (time_t *) malloc (sizeof (time_t) * 3)))
	    LeaveText ("cannot alloc memory for wait avg arrays", 1);
    }

    /*
    **  read nlist symbols, open /dev/kmem, /dev/mem, /dev/swap,
    **  initialize detail environment
    **  (all of these must occur after curses init)
    */
    SetupPaging ();
    (void) GetSymbols (kmemSymbols, 1);
    DetailInit ();
    DrwMainFrame ();

    ReadSysInfo();
    SaveSysInfo ();
    SaveCpuAndWait ();

    nowTime = GetTimeMsec ();

    /*
    **  initialize static display (literals)
    */
    DrwCpuScale (wscr, CPUSCALE_TLY, 0);
    DrwWaitScale (wscr, WAITSCALE_TLY, 0);
    DrwSysInfo (wscr, PER_SEC_TLY, 0);

    if (LINES > 38 && initial_cmd == (chtype) 0)
	initial_cmd = (chtype) 'e';

    napms (CYCLEmsec);

    while (1)
    {
	startClock = GetTimeMsec ();
	lastTime   = nowTime;
	nowTime    = startClock;
	nowInSecs  = time (NULL);
	stat_period_msec = nowTime - lastTime;
	lcTime = localtime (&nowInSecs);

	/*
	**  get command from user
	*/
	if (!ReadUserCommand ())
	    break;

	/*
	**  display current time
	*/
	wmove (wscr, 0, getmaxx (wscr) - 8);
	use_cp (wscr, cpBANNER);
	wprintw (wscr, "%02d:%02d:%02d", lcTime->tm_hour, lcTime->tm_min, lcTime->tm_sec);

	/*
	**  get system statistics
	*/
	ReadSysInfo ();

	/*
	**  check if analysis is really valid
	*/
	CheckForValidity ();

	/*
	**  calculate cpu deltas
	*/
	for (i = 0; i < 5; i++)
	    if (i != CPU_WAIT)
		cpu_ticks[i] = cpudelta (i);

	/*
	**  calculate wait deltas
	*/
	cpu_ticks[CPU_WAIT] = 0;
	for (i = 0; i < 3; i++)
	    cpu_ticks[CPU_WAIT] += (wait_ticks[i] = waitdelta (i));

	/*
	**  calculate and display cpu and wait avg (period 1)
	*/
	total_ticks = DynCpuScale (
	    wscr, CPUSCALE_TLY + 1, CPUSCALE_SX, CPUSCALE_WIDTH, cpu_ticks
	);
	UpdWaitScale (
	    wscr, WAITSCALE_TLY + 1, WAITSCALE_SX, WAITSCALE_WIDTH, wait_ticks, total_ticks
	);

	SaveCpuAvg (cpu_ticks, total_ticks);
	SaveWaitAvg (wait_ticks, total_ticks);

	/*
	**  calculate and display cpu and wait avg (period * 5)
	*/
	GetCpuAvg (cpu_ticks, 5);
	total_ticks = DynCpuScale (
	    wscr, CPUSCALE_TLY + 2, CPUSCALE_SX, CPUSCALE_WIDTH, cpu_ticks
	);
	GetWaitAvg (wait_ticks, 5);
	UpdWaitScale (
	    wscr, WAITSCALE_TLY + 2, WAITSCALE_SX, WAITSCALE_WIDTH, wait_ticks, total_ticks
	);

	/*
	**  calculate and display cpu and wait avg (period * 10)
	*/
	GetCpuAvg (cpu_ticks, 10);
	total_ticks = DynCpuScale (
	    wscr, CPUSCALE_TLY + 3, CPUSCALE_SX, CPUSCALE_WIDTH, cpu_ticks
	);
	GetWaitAvg (wait_ticks, 10);
	UpdWaitScale (
	    wscr, WAITSCALE_TLY + 3, WAITSCALE_SX, WAITSCALE_WIDTH, wait_ticks, total_ticks
	);

	/*
	**  update the sysinfo entries
	*/
	UpdSysInfo ();
	TcpPreCalc ();
	IpPreCalc ();

	DspDetailPanel ();

	TcpPostCalc ();
	IpPostCalc ();

	if (initial_cmd)
	{
	    DetailCommand (initial_cmd);
	    initial_cmd = 0;
	}

	/*
	**  flush the page
	*/
	PanelFlush ();

#if defined(LOGGING)
	if (kvarLogging && !kvarSuspended)
	    kvar_log (lcTime, kvar_symbols);
#endif

	/*
	**  remember previous statistics for next delta
	*/
	if (infoMode == modeDelta)
	    SaveSysInfo ();

	SaveCpuAndWait ();

	endClock = GetTimeMsec () - startClock + 10;

	if (endClock < 0)
	    endClock = 0;
	else if (endClock >= CYCLEmsec)
	    endClock = CYCLEmsec;

	if (poll (pollFd, 1, CYCLEmsec - endClock) == -1)
	    LeaveText ("poll() returns an error on stdin", 255);
    }

    Leave (0);
    /*NOTREACHED*/
}
