/*
**
**                             S V R 4 M O N . H
**
**
**      Module:     general header file for svr4mon
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      23-OCT-1992 AV  Version 1.0
**      24-OCT-1992 AV  new public function: DispTimer()
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**
*/

#ifndef _svr4mon_h_
#define _svr4mon_h_

#include <sys/types.h>
#include <sys/sysinfo.h>
#include <curses.h>

#ifndef min
#   define min(a,b)     (((a) > (b)) ? (b) : (a))
#endif

#if defined(COLOR_BLACK)
#   define cBLK     COLOR_BLACK
#   define cBLU     COLOR_BLUE
#   define cGRN     COLOR_GREEN
#   define cCYN     COLOR_CYAN
#   define cRED     COLOR_RED
#   define cMAG     COLOR_MAGENTA
#   define cBRN     COLOR_YELLOW
#   define cWHT     COLOR_WHITE
#   define cGRY     COLOR_BLACK
#   define cLTB     COLOR_BLUE
#   define cLTG     COLOR_GREEN
#   define cLTC     COLOR_CYAN
#   define cLTR     COLOR_RED
#   define cLTM     COLOR_MAGENTA
#   define cYEL     COLOR_YELLOW
#   define cHIW     COLOR_WHITE
#endif

#if defined(COLOR_PAIR)
#   define use_cp(win,cp)   wattrset(win, (color_avail) ? color_attr[cp] : mono_attr[cp])
#else
#   define use_cp(win,cp)   wattrset(win, mono_attr[cp])
#endif

/* 
**  if color par numbers are changed, disputil.c
**  color_attr and mono_attr tables
**  must also be changed
*/
#define cpINFO		1	/* information field */
#define cpREVERSE	2	/* "reverse video" */
#define cpBANWARN	3	/* banner warning */
#define cpLOW		4	/* low/user/io */
#define cpMED		5	/* medium/kernel/pio */
#define cpHIGH		6	/* high/brk/swp */
#define cpBANNER	7	/* banner */
#define cpLIT		8	/* field literals */

extern int  color_avail;

#if defined(COLOR_PAIR)
extern long     color_attr[];
#endif

extern long     mono_attr[];
extern long     stat_period_msec;
extern long     hertz;

extern long     memMax;
extern long     memFree;
extern long     swapMax;
extern long     swapFree;

extern enum info_mode_e {
    modeDelta, modeAbsolute, modeRelative
} infoMode;

extern struct sysinfo   out_sysinfo;
extern struct vminfo    out_vminfo;

/*
**  This variable contains the cpu time information specified in
**  the sysinfo structure:
**      CPU_IDLE, CPU_USER, CPU_KERNEL, CPU_WAIT, CPU_SXBRK
**  as percentages.
*/
extern time_t   cpu_percent[5];

/*
**  This variable contains the wait time information specified in the
**  sysinfo structure:
**      W_IO, W_SWAP, W_PIO
**  as percentages.
*/
extern time_t   wait_percent[3];



extern void DispTimer (WINDOW *win, int y, int x, int max);


#define MSG_TLY             (LINES - 1)

#define CPUSCALE_TLY        1
#define CPUSCALE_SX         13
#define CPUSCALE_WIDTH      (COLS - 30)

#define WAITSCALE_TLY       5
#define WAITSCALE_SX        13
#define WAITSCALE_WIDTH     (COLS - 30)

#define PER_SEC_TLY         9
#define PER_SEC1_TLX        0
#define PER_SEC2_TLX        17
#define PER_SEC3_TLX        35
#define PER_SEC4_TLX        51
#define PER_SEC5_TLX        67

/* extra info area */
#define EXTRA_TLY           24
#define EXTRA1_TLX          0
#define EXTRA2_TLX          20
#define EXTRA3_TLX          45
#define EXTRA4_TLX          65

#endif
