/*
**
**                                T U N E . C
**
**
**      Module:     display of tuneable parameters
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV      Version X1.0, first definition
**      21-OCT-1992 AV      start port for ARCH_MOT_88k
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#include <curses.h>
#include <panel.h>

#include <sys/types.h>
#include <sys/tuneable.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

#define ADDR_tune       kmemSymbols[0].n_value

static struct nlist kmemSymbols[] =
{
    { "tune"                },
    { 0                     },
};

static int      noSyms;

void DspTune (WINDOW *win, int y, int x)
{
    struct tune     *tune;

    use_cp (win, cpBANNER);
    wmove (win, y++, x);
    waddstr (win, "------ Tune -----");

    if (noSyms)
    {
	use_cp (win, cpBANWARN);

	y += 4;
	wmove (win, y++, x);
	waddstr (win, "not available, sorry!!");
	return;
    }

    tune = KTUT (ADDR_tune, struct tune *);

#if !defined(S_MOTOROLA_88K) && !defined(S_UNIVEL)
    wmove (win, y++, x);
    DispStaticInt (win, "t_ageintvl ", "%6d", tune->t_ageinterval);
#endif

    wmove (win, y++, x);
    DispStaticInt (win, "t_fsflushr ", "%6d", tune->t_fsflushr);

#if !defined(S_MOTOROLA_88K) && !defined(S_UNIVEL)
    wmove (win, y++, x);
    DispStaticInt (win, "t_gpgshi   ", "%6d", tune->t_gpgshi);
#endif

    wmove (win, y++, x);
    DispStaticInt (win, "t_gpgslo   ", "%6d", tune->t_gpgslo);

    wmove (win, y++, x);
    DispStaticInt (win, "t_minarmem ", "%6d", tune->t_minarmem);

    wmove (win, y++, x);
    DispStaticInt (win, "t_minasmem ", "%6d", tune->t_minasmem);

    wmove (win, y++, x);
    DispStaticInt (win, "t_dmalimit ", "%6d", tune->t_dmalimit);

    wmove (win, y++, x);
    DispStaticInt (win, "t_flckrec  ", "%6d", tune->t_flckrec);

#if defined(S_UNIVEL)
    wmove (win, y++, x);
    DispStaticInt (win, "t_dmabase  ", "%6d", tune->t_dmabase);

    wmove (win, y++, x);
    DispStaticInt (win, "t_devnondma", "%6d", tune->t_devnondma);
#else
    wmove (win, y++, x);
    DispStaticInt (win, "t_minakmem ", "%6d", tune->t_minakmem);
#endif
}

void TuneInit ()
{
    noSyms = GetSymbols (kmemSymbols, 0);
}
